"""base.py"""

import httpx


class BaseInterface:
    """Base interface"""

    url_path: str


class BaseSyncInterface(BaseInterface):
    """Base Synchronous interface"""

    client: httpx.Client

    def __init__(self, http_client: httpx.Client) -> None:
        self.client = http_client


class BaseAsyncInterface(BaseInterface):
    """Base Asynchronous interface"""

    client: httpx.AsyncClient

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        self.client = http_client



"""fine_tuning.py"""

from typing import List, Literal, Optional

from pydantic import BaseModel, NonNegativeInt, PositiveFloat, PositiveInt

from minimax_client.entities.common import BareResponse


class HyperParameters(BaseModel):
    """Fine Tuning Hyper Parameters"""

    batch_size: Optional[PositiveInt] = None
    learning_rate_multiplier: Optional[PositiveFloat] = None
    n_epochs: Optional[PositiveInt] = None


class FineTuningJob(BaseModel):
    """Fine Tuning Job"""

    id: str
    created_at: PositiveInt
    fine_tuned_model: str
    hyperparameters: Optional[HyperParameters] = None
    model: str
    object: Literal["finetune.job"]
    organization_id: str
    result_files: List[str]
    status: str
    training_file: NonNegativeInt
    validation_file: NonNegativeInt


class FineTuningJobCreateResponse(BareResponse):
    """Fine Tuning Job Create Response"""

    finetune_job: FineTuningJob


class FineTuningJobListResponse(BareResponse):
    """Fine Tuning Job List Response"""

    job_list: List[FineTuningJob] = []
    has_more: bool


class FineTuningJobEvent(BaseModel):
    """Fine Tuning Job Event"""

    id: str
    created_at: PositiveInt
    level: str
    message: str
    object: str


class FineTuningJobEventListResponse(BareResponse):
    """Fine Tuning Job Event List Response"""

    event_list: List[FineTuningJobEvent] = []
    has_more: bool


class FineTuningModel(BaseModel):
    """Fine Tuning Model"""

    id: str
    created_at: PositiveInt
    object: Literal["finetune.model"]
    base_model: Literal["abab5.5-chat-240119", "abab5.5s-chat-240123"]


class FineTuningModelListResponse(BareResponse):
    """Fine Tuning Model List Response"""

    model_list: List[FineTuningModel] = []


class FineTuningModelRetrieveResponse(BareResponse):
    """Fine Tuning Model Retrieve Response"""

    model: FineTuningModel



"""audio.py"""

from typing import Literal, Optional

from pydantic import BaseModel, HttpUrl, NonNegativeFloat, NonNegativeInt

from minimax_client.entities.common import BareResponse


class AudioSpeechErrorResponse(BareResponse):
    """Audio Speech Response"""

    trace_id: str


class AudioSpeechExtraInfo(BaseModel):
    """Audio Speech Extra Info"""

    audio_length: NonNegativeInt
    audio_sample_rate: NonNegativeInt
    audio_size: NonNegativeInt
    bitrate: NonNegativeInt
    word_count: NonNegativeInt
    invisible_character_ratio: NonNegativeFloat
    usage_characters: NonNegativeInt


class AudioSpeechProResponse(BareResponse):
    """Audio Speech Pro Response"""

    trace_id: str
    audio_file: Optional[HttpUrl] = None
    subtitle_file: Optional[HttpUrl] = None
    extra_info: Optional[AudioSpeechExtraInfo] = None


class AudioSpeechLargeResponse(BareResponse):
    """Audio Speech Large Response"""

    task_id: int
    task_token: str
    file_id: int
    usage_characters: NonNegativeInt


class AudioSpeechLargeStatusResponse(BareResponse):
    """Audio Speech Large Status Response"""

    status: Literal["Processing", "Success", "Failed", "Expired", ""]


class AudioSpeechStreamChunk(BaseModel):
    """Audio Speech Stream Chunk"""

    audio: str  # HEX encoded
    status: Literal[1, 2]
    ced: str


class AudioSpeechStreamResponse(BareResponse):
    """Audio Speech Stream Response"""

    trace_id: str
    data: AudioSpeechStreamChunk
    extra_info: Optional[AudioSpeechExtraInfo] = None


class VoiceCloningResponse(BareResponse):
    """Voice Cloning Response"""

    input_sensitive: bool
    input_sensitive_type: int



"""thread.py"""

import json
from typing import Any, AsyncGenerator, Dict, Generator, List, Literal, Optional, Union

import httpx

from minimax_client.entities.thread import (
    MessageCreateResponse,
    MessageListResponse,
    MessageRetrieveResponse,
    RunCancelResponse,
    RunCreateResponse,
    RunListResponse,
    RunRetrieveResponse,
    RunStepListResponse,
    RunStepRetrieveResponse,
    RunSubmitToolOutputsResponse,
    RunUpdateResponse,
    StreamedRunMessageResponse,
    StreamedRunResponse,
    StreamedRunStepResponse,
    ThreadCreateResponse,
    ThreadRetrieveResponse,
    ThreadUpdateResponse,
)
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class Messages(BaseSyncInterface):
    """Synchronous Messages interface"""

    url_path = "threads/messages"

    def create(
        self,
        thread_id: str,
        content: str,
        role: Literal["user", "assistant"],
        file_ids: Optional[List[str]] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> MessageCreateResponse:
        """
        Create a new message

        Args:
            thread_id (str): The ID of the thread to create the message in
            content (str): The content of the message
            role (str): The role of the message
            file_ids (Optional[List[str]], optional):
                The IDs of the files attached to the message. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the message. Defaults to None.

        Returns:
            MessageCreateResponse: The response from the API
        """
        json_body: Dict[str, Any] = {
            "thread_id": thread_id,
            "content": content,
            "role": role,
        }

        if file_ids:
            json_body["file_ids"] = file_ids

        if metadata:
            json_body["metadata"] = metadata

        resp = self.client.post(url=f"{self.url_path}/add", json=json_body)

        return MessageCreateResponse(**resp.json())

    def retrieve(self, message_id: str, thread_id: str) -> MessageRetrieveResponse:
        """
        Retrieve general info of the given message

        Args:
            message_id (str): The ID of the message to retrieve
            thread_id (str): The ID of the thread the message belongs to

        Returns:
            MessageRetrieveResponse: The response from the API
        """
        resp = self.client.get(
            url=f"{self.url_path}/retrieve",
            params={"message_id": message_id, "thread_id": thread_id},
        )

        return MessageRetrieveResponse(**resp.json())

    def list(
        self,
        thread_id: str,
        limit: int = 20,
        order: Literal["asc", "desc"] = "desc",
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> MessageListResponse:
        """
        List all messages in the given thread

        Args:
            thread_id (str): The ID of the thread to list messages from

        Returns:
            MessageListResponse: The response from the API
        """
        params = {"thread_id": thread_id, "limit": limit, "order": order}

        if after:
            params["after"] = after

        if before:
            params["before"] = before

        resp = self.client.get(url=f"{self.url_path}/list", params=params)

        return MessageListResponse(**resp.json())


class AsyncMessages(BaseAsyncInterface, Messages):
    """Asynchronous Messages interface"""

    async def create(
        self,
        thread_id: str,
        content: str,
        role: Literal["user", "assistant"],
        file_ids: Optional[List[str]] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> MessageCreateResponse:
        """
        Create a new message

        Args:
            thread_id (str): The ID of the thread to create the message in
            content (str): The content of the message
            role (str): The role of the message
            file_ids (Optional[List[str]], optional):
                The IDs of the files attached to the message. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the message. Defaults to None.

        Returns:
            MessageCreateResponse: The response from the API
        """
        json_body: Dict[str, Any] = {
            "thread_id": thread_id,
            "content": content,
            "role": role,
        }

        if file_ids:
            json_body["file_ids"] = file_ids

        if metadata:
            json_body["metadata"] = metadata

        resp = await self.client.post(url=f"{self.url_path}/add", json=json_body)

        return MessageCreateResponse(**resp.json())

    async def retrieve(
        self, message_id: str, thread_id: str
    ) -> MessageRetrieveResponse:
        """
        Retrieve general info of the given message

        Args:
            message_id (str): The ID of the message to retrieve
            thread_id (str): The ID of the thread the message belongs to

        Returns:
            MessageRetrieveResponse: The response from the API
        """
        resp = await self.client.get(
            url=f"{self.url_path}/retrieve",
            params={"message_id": message_id, "thread_id": thread_id},
        )

        return MessageRetrieveResponse(**resp.json())

    async def list(
        self,
        thread_id: str,
        limit: int = 20,
        order: Literal["asc", "desc"] = "desc",
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> MessageListResponse:
        """
        List all messages in the given thread

        Args:
            thread_id (str): The ID of the thread to list messages from

        Returns:
            MessageListResponse: The response from the API
        """
        params = {"thread_id": thread_id, "limit": limit, "order": order}

        if after:
            params["after"] = after

        if before:
            params["before"] = before

        resp = await self.client.get(url=f"{self.url_path}/list", params=params)

        return MessageListResponse(**resp.json())


class RunSteps(BaseSyncInterface):
    """Synchronous Run Steps interface"""

    url_path = "threads/run_steps"

    def retrieve(
        self, step_id: str, thread_id: str, run_id: str
    ) -> RunStepRetrieveResponse:
        """
        Retrieve general info of the given run step

        Args:
            step_id (str): The ID of the run step to retrieve
            thread_id (str): The ID of the thread the run belongs to
            run_id (str): The ID of the run the step belongs to

        Returns:
            RunStepRetrieveResponse: The response from the API
        """
        params = {"step_id": step_id, "thread_id": thread_id, "run_id": run_id}

        resp = self.client.get(url=f"{self.url_path}/retrieve", params=params)

        return RunStepRetrieveResponse(**resp.json())

    def list(
        self,
        run_id: str,
        thread_id: str,
        limit: int = 20,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> RunStepListResponse:
        """
        Retrive all run steps of the given run

        Args:
            run_id (str): The ID of the run to list steps from
            thread_id (str): The ID of the thread the run belongs to
            limit (int):
                The maximum number of steps to return. Defaults to 20.
            after (Optional[str]):
                The ID of the step to start after. Defaults to None.
            before (Optional[str]):
                The ID of the step to end before. Defaults to None.

        Returns:
            RunStepListResponse: The response from the API
        """
        params = {
            "run_id": run_id,
            "thread_id": thread_id,
            "limit": limit,
        }

        if after:
            params["after"] = after

        if before:
            params["before"] = before

        resp = self.client.get(url=f"{self.url_path}/list", params=params)

        return RunStepListResponse(**resp.json())


class AsyncRunSteps(BaseAsyncInterface, RunSteps):
    """Asynchronous Run Steps interface"""

    async def retrieve(
        self, step_id: str, thread_id: str, run_id: str
    ) -> RunStepRetrieveResponse:
        """
        Retrieve general info of the given run step

        Args:
            step_id (str): The ID of the run step to retrieve
            thread_id (str): The ID of the thread the run belongs to
            run_id (str): The ID of the run the step belongs to

        Returns:
            RunStepRetrieveResponse: The response from the API
        """
        params = {"step_id": step_id, "thread_id": thread_id, "run_id": run_id}

        resp = await self.client.get(url=f"{self.url_path}/retrieve", params=params)

        return RunStepRetrieveResponse(**resp.json())

    async def list(
        self,
        run_id: str,
        thread_id: str,
        limit: int = 20,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> RunStepListResponse:
        """
        Retrive all run steps of the given run

        Args:
            run_id (str): The ID of the run to list steps from
            thread_id (str): The ID of the thread the run belongs to
            limit (int):
                The maximum number of steps to return. Defaults to 20.
            after (Optional[str]):
                The ID of the step to start after. Defaults to None.
            before (Optional[str]):
                The ID of the step to end before. Defaults to None.

        Returns:
            RunStepListResponse: The response from the API
        """
        params = {
            "run_id": run_id,
            "thread_id": thread_id,
            "limit": limit,
        }

        if after:
            params["after"] = after

        if before:
            params["before"] = before

        resp = await self.client.get(url=f"{self.url_path}/list", params=params)

        return RunStepListResponse(**resp.json())


class Runs(BaseSyncInterface):
    """Synchronous Run interface"""

    url_path = "threads/run"
    steps: RunSteps

    def __init__(self, http_client: httpx.Client) -> None:
        super().__init__(http_client=http_client)
        self.steps = RunSteps(http_client=http_client)

    def create(
        self,
        thread_id: str,
        assistant_id: str,
        instructions: Optional[str] = None,
        model: Optional[
            Literal[
                "abab6-chat",
                "abab5.5-chat",
                "abab5.5-chat-240131",
                "abab5.5s-chat",
                "abab5.5s-chat-240123",
            ]
        ] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> RunCreateResponse:
        """
        Create a non-stream run

        Args:
            thread_id (str): The ID of the thread to create the run for
            assistant_id (str): The ID of the assistant to run
            instructions (Optional[str], optional):
                The instructions of the run. Defaults to None.
            model (
                Optional[
                    Literal[
                        "abab6-chat", "abab5.5-chat", "abab5.5-chat-240131",
                        "abab5.5s-chat", "abab5.5s-chat-240123"
                    ]
                ],
                optional
            ):
                The model to use for the run. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the run. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the run. Defaults to None.

        Returns:
            RunCreateResponse: The response from the API
        """
        json_body: Dict[str, Any] = {
            "thread_id": thread_id,
            "assistant_id": assistant_id,
        }

        if instructions:
            json_body["instructions"] = instructions

        if model:
            json_body["model"] = model

        if tools:
            json_body["tools"] = tools

        if metadata:
            json_body["metadata"] = metadata

        resp = self.client.post(url=f"{self.url_path}/create", json=json_body)

        return RunCreateResponse(**resp.json())

    def stream(  # noqa: C901
        self,
        *,
        stream_mode: Literal[1, 2],
        thread_id: str,
        assistant_id: str,
        messages: List[Dict[str, Union[int, str, List[str], Dict]]],
        model: Optional[
            Literal[
                "abab6-chat",
                "abab5.5-chat",
                "abab5.5-chat-240131",
                "abab5.5s-chat",
                "abab5.5s-chat-240123",
            ]
        ] = None,
        t2a_option: Optional[Dict[str, str]] = None,
        instructions: Optional[str] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> Generator[
        Union[StreamedRunMessageResponse, StreamedRunStepResponse, StreamedRunResponse],
        None,
        None,
    ]:
        """
        Create a stream run

        Args:
            stream_mode (Literal[1, 2]):
                The stream mode to use.
                1 for TEXT_STREAM,
                2 for TEXT_AND_AUDIO_STREAM (In this mode, the input could be audio)
            thread_id (str): The ID of the thread to create the run for
            assistant_id (str): The ID of the assistant to run
            messages (List[Dict[str, Union[int, str, List[str], Dict]]]):
                The messages to use for the run
            model (
                Optional[
                    Literal[
                        "abab6-chat", "abab5.5-chat", "abab5.5-chat-240131",
                        "abab5.5s-chat", "abab5.5s-chat-240123"
                    ]
                ]
            ):
                The model to use for the run.
                If not specified (ie. None), the model of the assistant will be used.
            t2a_option (Optional[Dict[str, str]], optional):
                T2A option to use for the run. Defaults to None.
            instructions (Optional[str], optional):
                The instructions of the run. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the run. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the run. Defaults to None.

        Returns:
            Generator[Union[
                StreamedRunMessageResponse, StreamedRunStepResponse, StreamedRunResponse
            ], None, None]: The response from the API
        """
        json_body = {
            "stream": stream_mode,
            "thread_id": thread_id,
            "assistant_id": assistant_id,
            "messages": messages,
        }

        if model:
            json_body["model"] = model

        if t2a_option:
            json_body["t2a_option"] = t2a_option

        if instructions:
            json_body["instructions"] = instructions

        if tools:
            json_body["tools"] = tools

        if metadata:
            json_body["metadata"] = metadata

        with self.client.stream(
            method="post", url=f"{self.url_path}/create_stream", json=json_body
        ) as resp:
            for data_text in resp.iter_text():
                if not data_text.startswith("data:"):
                    continue

                data_json = json.loads(data_text.split("data: ", 2)[1])

                if not data_json.get("data"):
                    continue

                if (object_tag := data_json["data"]["object"]) == "run":
                    yield StreamedRunResponse(**data_json)
                elif object_tag == "message":
                    yield StreamedRunMessageResponse(**data_json)
                elif object_tag == "run step":
                    yield StreamedRunStepResponse(**data_json)

    def retrieve(self, run_id: str, thread_id: str) -> RunRetrieveResponse:
        """
        Retrieve general info of the given run

        Args:
            run_id (str): The ID of the run to retrieve
            thread_id (str): The ID of the thread the run belongs to

        Returns:
            RunRetrieveResponse: The response from the API
        """
        resp = self.client.get(
            url=f"{self.url_path}/retrieve",
            params={"run_id": run_id, "thread_id": thread_id},
        )

        return RunRetrieveResponse(**resp.json())

    def list(
        self,
        thread_id: str,
        limit: int = 20,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> RunListResponse:
        """
        List all runs in the given thread

        Args:
            thread_id (str): The ID of the thread to list runs from
            limit (int, optional): The maximum number of runs to return. Defaults to 20.
            after (Optional[str], optional):
                The ID of the run to start from. Defaults to None.
            before (Optional[str], optional):
                The ID of the run to end at. Defaults to None.

        Returns:
            RunListResponse: The response from the API
        """
        params = {"thread_id": thread_id, "limit": limit}

        if after:
            params["after"] = after

        if before:
            params["before"] = before

        resp = self.client.get(url=f"{self.url_path}/list", params=params)

        return RunListResponse(**resp.json())

    def update(
        self, run_id: str, thread_id: str, metadata: Optional[Dict[str, str]] = None
    ) -> RunUpdateResponse:
        """
        Update general info of the given run

        Args:
            run_id (str): The ID of the run to update
            thread_id (str): The ID of the thread the run belongs to
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the run. Defaults to None.

        Returns:
            RunUpdateResponse: The response from the API
        """
        json_body: Dict[str, Any] = {
            "run_id": run_id,
            "thread_id": thread_id,
        }

        if metadata:
            json_body["metadata"] = metadata

        resp = self.client.post(url=f"{self.url_path}/modify", json=json_body)

        return RunUpdateResponse(**resp.json())

    def cancel(self, run_id: str, thread_id: str) -> RunCancelResponse:
        """
        Cancel an existing run

        Args:
            run_id (str): The ID of the run to cancel
            thread_id (str): The ID of the thread the run belongs to

        Returns:
            RunCancelResponse: The response from the API
        """
        json_body = {
            "run_id": run_id,
            "thread_id": thread_id,
        }

        resp = self.client.post(url=f"{self.url_path}/cancel", json=json_body)

        return RunCancelResponse(**resp.json())

    def submit_tool_outputs(
        self, run_id: str, thread_id: str, tool_outputs: List[Dict[str, str]]
    ) -> RunSubmitToolOutputsResponse:
        """
        Submit the outputs of tools to the given run

        Args:
            run_id (str): The ID of the run to submit
            thread_id (str): The ID of the thread the run belongs to
            tool_outputs (List[Dict[str, str]]): The outputs of the tools

        Returns:
            RunSubmitToolOutputsResponse: The response from the API
        """
        json_body = {
            "run_id": run_id,
            "thread_id": thread_id,
            "tool_outputs": tool_outputs,
        }

        resp = self.client.post(
            url=f"{self.url_path}/submit_tool_outputs", json=json_body
        )

        return RunSubmitToolOutputsResponse(**resp.json())


class AsyncRuns(BaseAsyncInterface, Runs):
    """Asynchronous Run interface"""

    steps: AsyncRunSteps

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        super().__init__(http_client=http_client)
        self.steps = AsyncRunSteps(http_client=http_client)

    async def create(
        self,
        thread_id: str,
        assistant_id: str,
        instructions: Optional[str] = None,
        model: Optional[
            Literal[
                "abab6-chat",
                "abab5.5-chat",
                "abab5.5-chat-240131",
                "abab5.5s-chat",
                "abab5.5s-chat-240123",
            ]
        ] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> RunCreateResponse:
        """
        Create a non-stream run

        Args:
            thread_id (str): The ID of the thread to create the run for
            assistant_id (str): The ID of the assistant to run
            instructions (Optional[str], optional):
                The instructions of the run. Defaults to None.
            model (
                Optional[
                    Literal[
                        "abab6-chat", "abab5.5-chat", "abab5.5-chat-240131",
                        "abab5.5s-chat", "abab5.5s-chat-240123"
                    ]
                ],
                optional
            ):
                The model to use for the run. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the run. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the run. Defaults to None.

        Returns:
            RunCreateResponse: The response from the API
        """
        json_body: Dict[str, Any] = {
            "thread_id": thread_id,
            "assistant_id": assistant_id,
        }

        if instructions:
            json_body["instructions"] = instructions

        if model:
            json_body["model"] = model

        if tools:
            json_body["tools"] = tools

        if metadata:
            json_body["metadata"] = metadata

        resp = await self.client.post(url=f"{self.url_path}/create", json=json_body)

        return RunCreateResponse(**resp.json())

    async def stream(  # noqa: C901
        self,
        *,
        stream_mode: Literal[1, 2],
        thread_id: str,
        assistant_id: str,
        messages: List[Dict[str, Union[int, str, List[str], Dict]]],
        model: Optional[
            Literal[
                "abab6-chat",
                "abab5.5-chat",
                "abab5.5-chat-240131",
                "abab5.5s-chat",
                "abab5.5s-chat-240123",
            ]
        ] = None,
        t2a_option: Optional[Dict[str, str]] = None,
        instructions: Optional[str] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        metadata: Optional[Dict[str, str]] = None,
    ) -> AsyncGenerator[
        Union[StreamedRunMessageResponse, StreamedRunStepResponse, StreamedRunResponse],
        None,
    ]:
        """
        Create a stream run

        Args:
            stream_mode (Literal[1, 2]):
                The stream mode to use.
                1 for TEXT_STREAM,
                2 for TEXT_AND_AUDIO_STREAM (In this mode, the input could be audio)
            thread_id (str): The ID of the thread to create the run for
            assistant_id (str): The ID of the assistant to run
            messages (List[Dict[str, Union[int, str, List[str], Dict]]]):
                The messages to use for the run
            model (
                Optional[
                    Literal[
                        "abab6-chat", "abab5.5-chat", "abab5.5-chat-240131",
                        "abab5.5s-chat", "abab5.5s-chat-240123"
                    ]
                ]
            ):
                The model to use for the run.
                If not specified (ie. None), the model of the assistant will be used.
            t2a_option (Optional[Dict[str, str]], optional):
                T2A option to use for the run. Defaults to None.
            instructions (Optional[str], optional):
                The instructions of the run. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the run. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the run. Defaults to None.

        Returns:
            AsyncGenerator[Union[
                StreamedRunMessageResponse, StreamedRunStepResponse, StreamedRunResponse
            ], None]: The response from the API
        """
        json_body = {
            "stream": stream_mode,
            "thread_id": thread_id,
            "assistant_id": assistant_id,
            "messages": messages,
        }

        if model:
            json_body["model"] = model

        if t2a_option:
            json_body["t2a_option"] = t2a_option

        if instructions:
            json_body["instructions"] = instructions

        if tools:
            json_body["tools"] = tools

        if metadata:
            json_body["metadata"] = metadata

        async with self.client.stream(
            method="post", url=f"{self.url_path}/create_stream", json=json_body
        ) as resp:
            async for data_text in resp.aiter_text():
                if not data_text.startswith("data:"):
                    continue

                data_json = json.loads(data_text.split("data: ", 2)[1])

                if not data_json.get("data"):
                    continue

                if (object_tag := data_json["data"]["object"]) == "run":
                    yield StreamedRunResponse(**data_json)
                elif object_tag == "message":
                    yield StreamedRunMessageResponse(**data_json)
                elif object_tag == "run step":
                    yield StreamedRunStepResponse(**data_json)

    async def retrieve(self, run_id: str, thread_id: str) -> RunRetrieveResponse:
        """
        Retrieve general info of the given run

        Args:
            run_id (str): The ID of the run to retrieve
            thread_id (str): The ID of the thread the run belongs to

        Returns:
            RunRetrieveResponse: The response from the API
        """
        resp = await self.client.get(
            url=f"{self.url_path}/retrieve",
            params={"run_id": run_id, "thread_id": thread_id},
        )

        return RunRetrieveResponse(**resp.json())

    async def list(
        self,
        thread_id: str,
        limit: int = 20,
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> RunListResponse:
        """
        List all runs in the given thread

        Args:
            thread_id (str): The ID of the thread to list runs from
            limit (int, optional): The maximum number of runs to return. Defaults to 20.
            after (Optional[str], optional):
                The ID of the run to start from. Defaults to None.
            before (Optional[str], optional):
                The ID of the run to end at. Defaults to None.

        Returns:
            RunListResponse: The response from the API
        """
        params = {"thread_id": thread_id, "limit": limit}

        if after:
            params["after"] = after

        if before:
            params["before"] = before

        resp = await self.client.get(url=f"{self.url_path}/list", params=params)

        return RunListResponse(**resp.json())

    async def update(
        self, run_id: str, thread_id: str, metadata: Optional[Dict[str, str]] = None
    ) -> RunUpdateResponse:
        """
        Update general info of the given run

        Args:
            run_id (str): The ID of the run to update
            thread_id (str): The ID of the thread the run belongs to
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the run. Defaults to None.

        Returns:
            RunUpdateResponse: The response from the API
        """
        json_body: Dict[str, Any] = {
            "run_id": run_id,
            "thread_id": thread_id,
        }

        if metadata:
            json_body["metadata"] = metadata

        resp = await self.client.post(url=f"{self.url_path}/modify", json=json_body)

        return RunUpdateResponse(**resp.json())

    async def cancel(self, run_id: str, thread_id: str) -> RunCancelResponse:
        """
        Cancel an existing run

        Args:
            run_id (str): The ID of the run to cancel
            thread_id (str): The ID of the thread the run belongs to

        Returns:
            RunCancelResponse: The response from the API
        """
        json_body = {
            "run_id": run_id,
            "thread_id": thread_id,
        }

        resp = await self.client.post(url=f"{self.url_path}/cancel", json=json_body)

        return RunCancelResponse(**resp.json())

    async def submit_tool_outputs(
        self, run_id: str, thread_id: str, tool_outputs: List[Dict[str, str]]
    ) -> RunSubmitToolOutputsResponse:
        """
        Submit the outputs of tools to the given run

        Args:
            run_id (str): The ID of the run to submit
            thread_id (str): The ID of the thread the run belongs to
            tool_outputs (List[Dict[str, str]]): The outputs of the tools

        Returns:
            RunSubmitToolOutputsResponse: The response from the API
        """
        json_body = {
            "run_id": run_id,
            "thread_id": thread_id,
            "tool_outputs": tool_outputs,
        }

        resp = await self.client.post(
            url=f"{self.url_path}/submit_tool_outputs", json=json_body
        )

        return RunSubmitToolOutputsResponse(**resp.json())


class Threads(BaseSyncInterface):
    """Synchronous Threads interface"""

    url_path = "threads"
    messages: Messages
    runs: Runs

    def __init__(self, http_client: httpx.Client) -> None:
        super().__init__(http_client=http_client)
        self.messages = Messages(http_client=http_client)
        self.runs = Runs(http_client=http_client)

    def create(self, metadata: Optional[Dict[str, str]] = None) -> ThreadCreateResponse:
        """
        Create a new thread

        Args:
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the thread. Defaults to None.

        Returns:
            ThreadCreateResponse: The created thread
        """
        json_body = {}

        if metadata:
            json_body["metadata"] = metadata

        resp = self.client.post(f"{self.url_path}/create", json=json_body)

        return ThreadCreateResponse(**resp.json())

    def retrieve(self, thread_id: str) -> ThreadRetrieveResponse:
        """
        Retrieve general info of the given thread

        Args:
            thread_id (str): The ID of the thread to retrieve

        Returns:
            ThreadRetrieveResponse: The response from the API
        """
        resp = self.client.get(
            f"{self.url_path}/retrieve", params={"thread_id": thread_id}
        )

        return ThreadRetrieveResponse(**resp.json())

    def update(
        self, thread_id: str, metadata: Optional[Dict[str, str]] = None
    ) -> ThreadUpdateResponse:
        """
        Update general info of the given thread

        Args:
            thread_id (str): The ID of the thread to update
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the thread. Defaults to None.

        Returns:
            ThreadUpdateResponse: The updated thread
        """
        json_body: Dict[str, Any] = {"id": thread_id}

        if metadata:
            json_body["metadata"] = metadata

        resp = self.client.post(f"{self.url_path}/modify", json=json_body)

        return ThreadUpdateResponse(**resp.json())


class AsyncThreads(BaseAsyncInterface, Threads):
    """Asynchronous Threads interface"""

    messages: AsyncMessages
    runs: AsyncRuns

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        super().__init__(http_client=http_client)
        self.messages = AsyncMessages(http_client=http_client)
        self.runs = AsyncRuns(http_client=http_client)

    async def create(
        self, metadata: Optional[Dict[str, str]] = None
    ) -> ThreadCreateResponse:
        """
        Create a new thread

        Args:
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the thread. Defaults to None.

        Returns:
            ThreadCreateResponse: The created thread
        """
        json_body = {}

        if metadata:
            json_body["metadata"] = metadata

        resp = await self.client.post(f"{self.url_path}/create", json=json_body)

        return ThreadCreateResponse(**resp.json())

    async def retrieve(self, thread_id: str) -> ThreadRetrieveResponse:
        """
        Retrieve general info of the given thread

        Args:
            thread_id (str): The ID of the thread to retrieve

        Returns:
            ThreadRetrieveResponse: The response from the API
        """
        resp = await self.client.get(
            f"{self.url_path}/retrieve", params={"thread_id": thread_id}
        )

        return ThreadRetrieveResponse(**resp.json())

    async def update(
        self, thread_id: str, metadata: Optional[Dict[str, str]] = None
    ) -> ThreadUpdateResponse:
        """
        Update general info of the given thread

        Args:
            thread_id (str): The ID of the thread to update
            metadata (Optional[Dict[str, str]], optional):
                The metadata of the thread. Defaults to None.

        Returns:
            ThreadUpdateResponse: The updated thread
        """
        json_body: Dict[str, Any] = {"id": thread_id}

        if metadata:
            json_body["metadata"] = metadata

        resp = await self.client.post(f"{self.url_path}/modify", json=json_body)

        return ThreadUpdateResponse(**resp.json())



"""chat_completion.py"""

import json
from http import HTTPStatus
from typing import Any, AsyncGenerator, Dict, Generator, List, Optional, Union

import httpx

from minimax_client.entities.chat_completion import ChatCompletionResponse
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class ChatCompletions(BaseSyncInterface):
    """Synchronous Chat Completions interface"""

    url_path: str = "text/chatcompletion_v2"

    def create(
        self,
        *,
        messages: List[Dict[str, Union[str, List[Dict[str, Any]]]]],
        model: str = "abab5.5s-chat",
        max_tokens: int = 256,
        temperature: float = 0.9,
        top_p: float = 0.95,
        stream: bool = False,
        tool_choice: str = "auto",
        tools: Optional[List[Dict[str, Union[str, Dict[str, str]]]]] = None,
    ) -> Union[ChatCompletionResponse, Generator[ChatCompletionResponse, None, None]]:
        """
        Create a new chat completion request.

        Requests can be sent in or not in stream by setting the "stream" parameter.
        Streaming requests return a generator that yields ResponseEntities,
        while non-streaming requests return a single ResponseEntity.

        Args:
            messages (list[dict[str, Union[str, list[dict[str, Any]]]]]):
                The messages to generate responses to
            model (str, optional):
                The chatbot model to use. Defaults to "abab5.5s-chat".
            max_tokens (int, optional):
                The maximum number of tokens to generate. Defaults to 256.
            temperature (float, optional):
                The temperature of the chatbot's responses. Defaults to 0.9.
            top_p (float, optional):
                The top_p of the chatbot's responses. Defaults to 0.95.
            stream (bool, optional):
                Whether to stream the responses or not. Defaults to False.
            tool_choice (str, optional):
                The tool choice mode. Defaults to "auto".
            tools (Optional[list[dict[str, Union[str, dict[str, str]]]]], optional):
                The tools to use. Defaults to None.

        Returns:
            ChatCompletionResponse | Generator[ChatCompletionResponse, None, None]:
                The response from the API or a generator of responses
        """
        json_body = {
            "messages": messages,
            "model": model,
            "max_tokens": max_tokens,
            "temperature": temperature,
            "top_p": top_p,
            "stream": stream,
            "tool_choice": tool_choice,
            "tools": tools if tools else [],
        }

        if not stream:
            resp = self.client.post(url=self.url_path, json=json_body)
            return self._build_response(resp=resp)

        return self._build_stream_response(json_body=json_body)

    def _build_response(self, resp: httpx.Response) -> ChatCompletionResponse:
        """
        Builds a Chat Completion response from an HTTP response

        Args:
            resp (httpx.Response): The HTTP response

        Raises:
            Exception: If the HTTP response is not OK or if parsing the response fails

        Returns:
            ChatCompletionResponse: The response from the API
        """
        if resp.status_code != HTTPStatus.OK:
            raise Exception(f"status: {resp.status_code}; {resp.text}")

        try:
            chat_response = ChatCompletionResponse(**resp.json())
        except Exception as e:
            raise Exception(f"Failed to parse response: {e}")  # noqa: B904

        return chat_response

    def _build_stream_response(
        self, json_body: Dict[str, Any]
    ) -> Generator[ChatCompletionResponse, None, None]:
        """
        Builds a stream of Chat Completion responses from an HTTP response

        Args:
            json_body (dict): The JSON body of the request

        Yields:
            ChatCompletionResponse: The response from the API
        """
        with self.client.stream(
            method="post", url=self.url_path, json=json_body
        ) as resp:
            if resp.status_code != HTTPStatus.OK:
                raise Exception(f"status: {resp.status_code}; {resp.text}")

            for data_text in resp.iter_text():
                data_json = json.loads(data_text.split("data: ", 2)[1])

                yield ChatCompletionResponse(**data_json)

                # If the stream is finished, break out of the loop
                if "finish_reason" in data_json["choices"][0]:
                    break


class AsyncChatCompletions(BaseAsyncInterface, ChatCompletions):
    """Asynchronous Chat Completions interface"""

    async def create(
        self,
        *,
        messages: List[Dict[str, Union[str, List[Dict[str, Any]]]]],
        model: str = "abab5.5s-chat",
        max_tokens: int = 256,
        temperature: float = 0.9,
        top_p: float = 0.95,
        stream: bool = False,
        tool_choice: str = "auto",
        tools: Optional[List[Dict[str, Union[str, Dict[str, str]]]]] = None,
    ) -> Union[ChatCompletionResponse, AsyncGenerator[ChatCompletionResponse, None]]:
        """
        Create a new chat completion for the given messages.

        Args:
            messages (list[dict[str, Union[str, list[dict[str, Any]]]]]):
                The messages to generate responses to
            model (str, optional):
                The chatbot model to use. Defaults to "abab5.5s-chat".
            max_tokens (int, optional):
                The maximum number of tokens to generate. Defaults to 256.
            temperature (float, optional):
                The temperature of the chatbot's responses. Defaults to 0.9.
            top_p (float, optional):
                The top_p of the chatbot's responses. Defaults to 0.95.
            stream (bool, optional):
                Whether to stream the responses or not. Defaults to False.
            tool_choice (str, optional):
                The tool choice mode. Defaults to "auto".
            tools (Optional[list[dict[str, Union[str, dict[str, str]]]]], optional):
                The tools to use. Defaults to None.

        Returns:
            ChatCompletionResponse | AsyncGenerator[ChatCompletionResponse, None]:
                The response from the API or a generator of responses
        """
        json_body = {
            "messages": messages,
            "model": model,
            "max_tokens": max_tokens,
            "temperature": temperature,
            "top_p": top_p,
            "stream": stream,
            "tool_choice": tool_choice,
            "tools": tools if tools else [],
        }

        if not stream:
            resp = await self.client.post(url=self.url_path, json=json_body)
            return self._build_response(resp=resp)

        return self._build_stream_response(json_body=json_body)

    async def _build_stream_response(
        self, json_body: Dict[str, Any]
    ) -> AsyncGenerator[ChatCompletionResponse, None]:
        """
        Builds a stream of Chat Completion responses from an HTTP response

        Args:
            json_body (dict): The JSON body of the request

        Yields:
            ChatCompletionResponse: The response from the API
        """
        async with self.client.stream(
            method="post", url=self.url_path, json=json_body
        ) as resp:
            if resp.status_code != HTTPStatus.OK:
                raise Exception(f"status: {resp.status_code}; {resp.text}")

            async for data in resp.aiter_text():
                json_body = json.loads(data.split("data: ", 2)[1])

                yield ChatCompletionResponse(**json_body)

                # If the stream is finished, break out of the loop
                if "finish_reason" in json_body["choices"][0]:
                    break


class Chat:
    """Synchronous Chat interface"""

    completions: ChatCompletions

    def __init__(self, http_client: httpx.Client) -> None:
        """
        Initializes the Chat interface

        Args:
            http_client (httpx.Client): The HTTP client to use
        """
        self.completions = ChatCompletions(http_client=http_client)


class AsyncChat:
    """Asynchronous Chat interface"""

    completions: AsyncChatCompletions

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        """
        Initializes the AsyncChat interface

        Args:
            http_client (httpx.AsyncClient): The HTTP client to use
        """
        self.completions = AsyncChatCompletions(http_client=http_client)



"""chat_completion.py"""

from typing import List, Literal, Optional

from pydantic import BaseModel, NonNegativeInt

from minimax_client.entities.common import BareResponse


class ChoiceMessageToolCallFunction(BaseModel):
    """Chat Completion Choice Message ToolCall Function"""

    name: str
    arguments: str


class ChoiceMessageToolCall(BaseModel):
    """Chat Completion Choice Message ToolCall"""

    id: str
    type: Literal["function"]
    function: ChoiceMessageToolCallFunction


class ChoiceMessage(BaseModel):
    """Chat Completion Choice Message"""

    role: Literal["assistant", "user", "system", "tool"]
    content: Optional[str] = None
    tool_calls: Optional[List[ChoiceMessageToolCall]] = None


class Choice(BaseModel):
    """Chat Completion Choice"""

    index: NonNegativeInt
    message: Optional[ChoiceMessage] = None
    delta: Optional[ChoiceMessage] = None
    finish_reason: Optional[Literal["length", "stop", "tool_calls"]] = None


class Usage(BaseModel):
    """Chat Completion Response Usage"""

    total_tokens: NonNegativeInt


class ChatCompletionResponse(BareResponse):
    """Chat Completion Response"""

    id: str
    choices: List[Choice]
    created: int
    model: Literal["abab5.5s-chat", "abab5.5-chat", "abab6-chat"]
    object: Literal["chat.completion", "chat.completion.chunk"]
    usage: Optional[Usage] = None



"""assistant.py"""

from typing import Dict, List, Literal, Optional

from pydantic import BaseModel, PositiveInt

from minimax_client.entities.common import BareResponse


class AssistantToolFunctionParameters(BaseModel):
    """Assistant Tool Function Parameters"""

    type: Literal["object"]
    required: List[str]
    properties: Dict[str, Dict]


class AssistantToolFunction(BaseModel):
    """Assistant Tool Function"""

    description: str
    name: str
    parameters: Optional[AssistantToolFunctionParameters] = None


class AssistantTool(BaseModel):
    """Assistant Tool"""

    type: Literal["code_interpreter", "retrieval", "function", "web_search"]
    function: Optional[AssistantToolFunction] = None


class AssistantT2AOption(BaseModel):
    """Assistant T2A Option"""

    model: str
    voice_id: str
    format: Literal["mp3", "flac", "pcm"] = "mp3"


class Assistant(BaseModel):
    """Assistant"""

    id: str
    object: Literal["assistant"]
    created_at: PositiveInt
    updated_at: Optional[PositiveInt] = None
    name: str
    description: str
    model: Literal[
        "abab6-chat",
        "abab5.5-chat",
        "abab5.5-chat-240131",
        "abab5.5s-chat",
        "abab5.5s-chat-240123",
    ]
    instructions: str
    tools: List[AssistantTool] = []
    file_ids: List[str] = []
    metadata: Dict = {}
    rolemeta: Dict
    status: str
    t2a_option: Optional[AssistantT2AOption] = None


class AssistantCreateResponse(BareResponse, Assistant):
    """Assistant Create Response"""


class AssistantRetrieveResponse(BareResponse, Assistant):
    """Assistant Retrieve Response"""


class AssistantUpdateResponse(BareResponse):
    """Assistant Update Response"""

    assistant: Assistant


class AssistantDeleteResponse(BareResponse):
    """Assistant Delete Response"""

    id: str
    object: Literal["assistant.deleted"]
    deleted: bool


class AssistantListResponse(BareResponse):
    """Assistant List Response"""

    object: Literal["list"]
    data: List[Assistant]
    has_more: bool
    first_id: str
    last_id: str


class AssistantFile(BaseModel):
    """Assistant File"""

    id: str
    object: Literal["assistant.file"]
    created_at: PositiveInt
    assistant_id: str


class AssistantFileCreateResponse(BareResponse, AssistantFile):
    """Assistant File Create Response"""


class AssistantFileRetrieveResponse(BareResponse, AssistantFile):
    """Assistant File Retrieve Response"""


class AssistantFileListResponse(BareResponse):
    """Assistant File List Response"""

    object: Literal["list"]
    data: List[AssistantFile]


class AssistantFileDeleteResponse(BareResponse):
    """Assistant File Delete Response"""

    file_id: str



"""embedding.py"""

from http import HTTPStatus
from typing import List, Union

import httpx

from minimax_client.entities.embedding import EmbeddingResponse
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class Embedding(BaseSyncInterface):
    """Synchronous Embedding interface"""

    url_path: str = "embeddings"

    def create(
        self,
        *,
        input: Union[str, List[str]],
        model: str = "embo-01",
        target: str = "db",
    ) -> EmbeddingResponse:
        """
        Create embeddings for the given input text(s).

        Args:
            input (str | list[str]): The input text(s) to embed.
            model (str, optional): The model to use. For now only embo-01 is available.
                Defaults to "embo-01".
            target (str, optional): The target scenario to use the embedding(s).
                Can be either "db" or "query". Defaults to "db".

        Returns:
            EmbeddingResponse: The response from the API.
        """
        if isinstance(input, str):
            input = [input]

        resp = self.client.post(
            url=self.url_path, json={"texts": input, "model": model, "type": target}
        )

        return self._build_response(resp=resp)

    def _build_response(self, resp: httpx.Response) -> EmbeddingResponse:
        """
        Builds an Embeddings response from an HTTP response

        Args:
            resp (httpx.Response): The HTTP response

        Raises:
            Exception: If the HTTP response is not OK or if parsing the response fails

        Returns:
            EmbeddingResponse: The response from the API
        """
        if resp.status_code != HTTPStatus.OK:
            raise Exception(f"status: {resp.status_code}; {resp.text}")

        try:
            response_entity = EmbeddingResponse(**resp.json())
        except Exception as e:
            raise Exception(f"Failed to parse response: {e}")  # noqa: B904

        return response_entity


class AsyncEmbedding(BaseAsyncInterface, Embedding):
    """Asynchronous Embedding interface"""

    client: httpx.AsyncClient

    async def create(
        self,
        *,
        input: Union[str, List[str]],
        model: str = "embo-01",
        target: str = "db",
    ) -> EmbeddingResponse:
        """
        Create embeddings for the given input text(s).

        Args:
            input (str | list[str]): The input text(s) to embed.
            model (str, optional): The model to use. For now only embo-01 is available.
                Defaults to "embo-01".
            target (str, optional): The target scenario to use the embedding(s).
                Can be either "db" or "query". Defaults to "db".

        Returns:
            EmbeddingResponse: The response from the API.
        """
        if isinstance(input, str):
            input = [input]

        resp = await self.client.post(
            url=self.url_path, json={"texts": input, "model": model, "type": target}
        )

        return self._build_response(resp=resp)



"""file.py"""

from typing import List, Optional

from pydantic import BaseModel, HttpUrl, NonNegativeInt

from minimax_client.entities.common import BareResponse


class File(BaseModel):
    """
    File entity

    purpose:
        retrieval -> pdf, docx, txt
        fine-tune -> jsonl
        voice_clone -> mp3, m4a, wav
        assistants -> refer to official documents
        role-recognition -> json, txt(with json content)
    """

    file_id: NonNegativeInt
    bytes: NonNegativeInt
    created_at: int
    filename: str
    purpose: str
    download_url: Optional[HttpUrl] = None


class FileCreateResponse(BareResponse):
    """File Create Response"""

    file: File


class FileListResponse(BareResponse):
    """File List Response"""

    files: List[File]


class FileRetriveResponse(BareResponse):
    """File Retrieve Response"""

    file: File


class FileRetrieveContentResponse(BareResponse):
    """File Retrieve Content Response"""

    content: bytes  # to be confirmed


class FileDeleteResponse(BareResponse):
    """File Delete Response"""

    file_id: NonNegativeInt



"""thread.py"""

from typing import Dict, List, Literal, Optional

from pydantic import BaseModel, HttpUrl, NonNegativeInt

from minimax_client.entities.assistant import AssistantTool
from minimax_client.entities.chat_completion import ChoiceMessageToolCall
from minimax_client.entities.common import BareResponse


class StreamedRunBaseResp(BaseModel):
    """Streamed Run Base Resp"""

    code: Optional[NonNegativeInt] = None
    message: Optional[str] = None


class Thread(BaseModel):
    """Thread"""

    id: str
    object: Literal["thread"]
    created_at: NonNegativeInt
    metadata: Dict[str, str] = {}
    updated_at: Optional[NonNegativeInt] = None


class ThreadCreateResponse(BareResponse, Thread):
    """Thread Create Response"""


class ThreadRetrieveResponse(BareResponse, Thread):
    """Thread Retrieve Response"""


class ThreadUpdateResponse(BareResponse):
    """Thread Update Response"""

    thread: Thread


class MessageTextContentAnnotationFileCitation(BaseModel):
    """Message Text Content Annotation File Citation"""

    file_id: str
    quote: str


class MessageTextContentAnnotationWebCitation(BaseModel):
    """Message Text Content Annotation Web Citation"""

    url: HttpUrl
    quote: str


class MessageTextContentAnnotation(BaseModel):
    """Message Text Content Annotation"""

    type: str
    text: str
    start_index: NonNegativeInt
    end_index: NonNegativeInt
    file_citation: Optional[MessageTextContentAnnotationFileCitation] = None
    web_citation: Optional[MessageTextContentAnnotationWebCitation] = None


class MessageTextContent(BaseModel):
    """Message Text Content"""

    value: str
    annotations: Optional[List[MessageTextContentAnnotation]] = None


class MessageImageFile(BaseModel):
    """Message Image File"""

    file_id: str


class MessageContent(BaseModel):
    """Message Content"""

    type: str
    text: Optional[MessageTextContent] = None
    image_file: Optional[MessageImageFile] = None


class Message(BaseModel):
    """Message"""

    id: str
    object: Literal["message"]
    created_at: NonNegativeInt
    thread_id: str
    role: str
    content: List[MessageContent] = []
    file_ids: Optional[List[str]] = None
    assistant_id: str
    run_id: str
    metadata: Optional[Dict[str, str]] = None
    updated_at: Optional[NonNegativeInt] = None


class MessageCreateResponse(BareResponse, Message):
    """Message Create Response"""


class MessageRetrieveResponse(BareResponse, Message):
    """Message Retrieve Response"""


class MessageListResponse(BareResponse):
    """Message List Response"""

    object: Literal["list"]
    data: List[Message]
    first_id: str
    last_id: str


class StreamedRunMessageResponse(BaseModel):
    """Streamed Run Message Response"""

    data: Message
    base_resp: StreamedRunBaseResp


class RunError(BaseModel):
    """Run Error"""

    code: str
    message: str


class RunRequiredActionSubmitToolOutputs(BaseModel):
    """Run Required Action Submit Tool Outputs"""

    tool_calls: List[ChoiceMessageToolCall]


class RunRequiredAction(BaseModel):
    """Run Required Action"""

    type: Literal["submit_tool_outputs", ""]
    submit_tool_outputs: Optional[RunRequiredActionSubmitToolOutputs] = None


class T2AOption(BaseModel):
    """T2A Option"""

    model: Literal["speech-01"]
    voice_id: str


class StreamedRunT2AControl(BaseModel):
    """Streamed Run T2A Control"""

    combine: bool
    model: Literal["speech-01"]
    oss: str
    return_bytes: bool
    text: str
    timbre_weights: Dict[str, float]


class Run(BaseModel):
    """Run"""

    id: str
    object: Literal["thread.run", "run"]
    assistant_id: str
    thread_id: str
    status: Literal[
        "queued",
        "started",
        "in_progress",
        "requires_action",
        "completed",
        "expired",
        "cancelled",
        "failed",
    ]
    created_at: NonNegativeInt
    started_at: Optional[NonNegativeInt] = None
    expires_at: Optional[NonNegativeInt] = None
    cancelled_at: Optional[NonNegativeInt] = None
    failed_at: Optional[NonNegativeInt] = None
    completed_at: Optional[NonNegativeInt] = None
    updated_at: Optional[NonNegativeInt] = None
    last_error: Optional[RunError] = None
    model: Literal[
        "abab6-chat",
        "abab5.5-chat",
        "abab5.5-chat-240131",
        "abab5.5s-chat",
        "abab5.5s-chat-240123",
    ]
    t2a_option: Optional[T2AOption] = None
    t2a_control: Optional[StreamedRunT2AControl] = None
    instructions: str
    tools: List[AssistantTool] = []
    file_ids: List[str] = []
    metadata: Optional[Dict[str, str]] = None
    required_action: Optional[RunRequiredAction] = None


class StreamedRunResponse(BaseModel):
    """Streamed Run Response"""

    data: Run
    base_resp: StreamedRunBaseResp


class RunCreateResponse(BareResponse, Run):
    """Run Create Response"""


class RunRetrieveResponse(BareResponse, Run):
    """Run Retrieve Response"""


class RunListResponse(BareResponse):
    """Run List Response"""

    object: Literal["list"]
    data: List[Run]


class RunUpdateResponse(BareResponse):
    """Run Update Response"""

    run: Run


class RunCancelResponse(BareResponse):
    """Run Cancel Response"""

    run: Optional[Run] = None


class RunSubmitToolOutputsResponse(BareResponse, Run):
    """Run Submit Tool Outputs Response"""


class RunStepDetailMessageCreation(BaseModel):
    """Run Step Detail Message Creation"""

    message_id: str


class RunStepDetailToolCallCodeInterpreterOutput(BaseModel):
    """Run Step Detail Tool Call Code Interpreter Output"""

    type: str
    logs: Optional[str] = None


class RunStepDetailToolCallCodeInterpreter(BaseModel):
    """Run Step Detail Tool Call Code Interpreter"""

    input: str
    outputs: List[RunStepDetailToolCallCodeInterpreterOutput]


class RunStepDetailToolCallWebSearch(BaseModel):
    """Run Step Detail Tool Call Web Search"""

    query: str = ""
    outputs: str = ""
    name: str = ""
    arguments: str = ""


class RunStepDetailToolCallRetrieval(BaseModel):
    """Run Step Detail Tool Call Retrieval"""

    query: str
    outputs: str


class RunStepDetailToolCallFuntion(BaseModel):
    """Run Step Detail Tool Call Function"""

    name: str
    arguments: str
    output: str


class RunStepDetailToolCall(BaseModel):
    """Run Step Detail Tool Call"""

    id: str
    type: Literal["code_interpreter", "web_search", "retrieval", "function"]
    code_interpreter: Optional[RunStepDetailToolCallCodeInterpreter] = None
    web_search: Optional[RunStepDetailToolCallWebSearch] = None
    retrieval: Optional[RunStepDetailToolCallRetrieval] = None
    function: Optional[RunStepDetailToolCallFuntion] = None


class RunStepDetail(BaseModel):
    """Run Step Detail"""

    type: Literal["message_creation", "tool_calls"]
    message_creation: Optional[RunStepDetailMessageCreation] = None
    tool_calls: Optional[List[RunStepDetailToolCall]] = None


class RunStep(BaseModel):
    """Run Step"""

    id: str
    object: Literal["thread.run.step", "run step"]
    run_id: str
    assistant_id: str
    thread_id: str
    type: Literal["message_creation", "tool_calls"]
    status: str
    created_at: NonNegativeInt
    expired_at: Optional[NonNegativeInt] = None
    cancelled_at: Optional[NonNegativeInt] = None
    failed_at: Optional[NonNegativeInt] = None
    completed_at: Optional[NonNegativeInt] = None
    last_error: Optional[RunError] = None
    step_details: RunStepDetail
    base_resp: Optional[Dict] = None


class RunStepRetrieveResponse(BareResponse, RunStep):
    """Run Step Retrieve response"""


class RunStepListResponse(BareResponse):
    """Run Step List response"""

    object: Literal["list"]
    data: List[RunStep]


class StreamedRunStepResponse(BaseModel):
    """Streamed Run Step Response"""

    data: RunStep
    base_resp: StreamedRunBaseResp



"""assistant.py"""

from typing import Any, Dict, List, Literal, Optional, Union

import httpx

from minimax_client.entities.assistant import (
    AssistantCreateResponse,
    AssistantDeleteResponse,
    AssistantFileCreateResponse,
    AssistantFileDeleteResponse,
    AssistantFileListResponse,
    AssistantFileRetrieveResponse,
    AssistantListResponse,
    AssistantRetrieveResponse,
    AssistantUpdateResponse,
)
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class AssistantFile(BaseSyncInterface):
    """Synchronous Assistant File interface"""

    url_path = "assistants/files"

    def create(self, assistant_id: str, file_id: str) -> AssistantFileCreateResponse:
        """
        Add an existing file to given assistant

        Args:
            assistant_id (str): The ID of the assistant
            file_id (str): The ID of the file to add to the assistant

        Returns:
            AssistantFileCreateResponse: The response from the API
        """
        resp = self.client.post(
            url=f"{self.url_path}/create",
            json={"assistant_id": assistant_id, "file_id": file_id},
        )

        return AssistantFileCreateResponse(**resp.json())

    def retrieve(
        self, assistant_id: str, file_id: str
    ) -> AssistantFileRetrieveResponse:
        """
        Retrieve info of a file associated with the given assistant

        Args:
            assistant_id (str): The ID of the assistant
            file_id (str): The ID of the file associated with the assistant

        Returns:
            AssistantFileRetrieveResponse: The response from the API
        """
        resp = self.client.get(
            url=f"{self.url_path}/retrieve",
            params={"assistant_id": assistant_id, "file_id": file_id},
        )

        return AssistantFileRetrieveResponse(**resp.json())

    def list(
        self,
        assistant_id: str,
        limit: int = 20,
        order: Literal["asc", "desc"] = "desc",
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> AssistantFileListResponse:
        """
        List files associated with the given assistant

        Args:
            assistant_id (str): The ID of the assistant
            limit (int): The number of files to return. Defaults to 20.
            order (str):
                The order of the list, could be "asc" or "desc". Defaults to "desc".
            after (Optional[str], optional):
                The ID of the file to start after. Defaults to None.
            before (Optional[str], optional):
                The ID of the file to end before. Defaults to None.

        Returns:
            AssistantFileListResponse: The response from the API
        """
        params = {"assistant_id": assistant_id, "limit": limit, "order": order}

        if after:
            params["after"] = after
        if before:
            params["before"] = before

        resp = self.client.get(url=f"{self.url_path}/list", params=params)

        return AssistantFileListResponse(**resp.json())

    def delete(self, assistant_id: str, file_id: str) -> AssistantFileDeleteResponse:
        """
        Delete a file associated with the given assistant

        Args:
            assistant_id (str): The ID of the assistant
            file_id (str):
                The ID of the file to delete that is associated with the assistant

        Returns:
            AssistantFileDeleteResponse: The response from the API
        """
        resp = self.client.post(
            url=f"{self.url_path}/delete",
            json={"assistant_id": assistant_id, "file_id": file_id},
        )

        return AssistantFileDeleteResponse(**resp.json())


class AsyncAssistantFile(BaseAsyncInterface, AssistantFile):
    """Asynchronous Assistant File interface"""

    async def create(
        self, assistant_id: str, file_id: str
    ) -> AssistantFileCreateResponse:
        """
        Add an existing file to given assistant

        Args:
            assistant_id (str): The ID of the assistant
            file_id (str): The ID of the file to add to the assistant

        Returns:
            AssistantFileCreateResponse: The response from the API
        """
        resp = await self.client.post(
            url=f"{self.url_path}/create",
            json={"assistant_id": assistant_id, "file_id": file_id},
        )

        return AssistantFileCreateResponse(**resp.json())

    async def retrieve(
        self, assistant_id: str, file_id: str
    ) -> AssistantFileRetrieveResponse:
        """
        Retrieve info of a file associated with the given assistant

        Args:
            assistant_id (str): The ID of the assistant
            file_id (str): The ID of the file associated with the assistant

        Returns:
            AssistantFileRetrieveResponse: The response from the API
        """
        resp = await self.client.get(
            url=f"{self.url_path}/retrieve",
            params={"assistant_id": assistant_id, "file_id": file_id},
        )

        return AssistantFileRetrieveResponse(**resp.json())

    async def list(
        self,
        assistant_id: str,
        limit: int = 20,
        order: Literal["asc", "desc"] = "desc",
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> AssistantFileListResponse:
        """
        List files associated with the given assistant

        Args:
            assistant_id (str): The ID of the assistant
            limit (int): The number of files to return. Defaults to 20.
            order (str):
                The order of the list, could be "asc" or "desc". Defaults to "desc".
            after (Optional[str], optional):
                The ID of the file to start after. Defaults to None.
            before (Optional[str], optional):
                The ID of the file to end before. Defaults to None.

        Returns:
            AssistantFileListResponse: The response from the API
        """
        params = {"assistant_id": assistant_id, "limit": limit, "order": order}

        if after:
            params["after"] = after
        if before:
            params["before"] = before

        resp = await self.client.get(url=f"{self.url_path}/list", params=params)

        return AssistantFileListResponse(**resp.json())

    async def delete(
        self, assistant_id: str, file_id: str
    ) -> AssistantFileDeleteResponse:
        """
        Delete a file associated with the given assistant

        Args:
            assistant_id (str): The ID of the assistant
            file_id (str):
                The ID of the file to delete that is associated with the assistant

        Returns:
            AssistantFileDeleteResponse: The response from the API
        """
        resp = await self.client.post(
            url=f"{self.url_path}/delete",
            json={"assistant_id": assistant_id, "file_id": file_id},
        )

        return AssistantFileDeleteResponse(**resp.json())


class Assistant(BaseSyncInterface):
    """Synchronous Assistants interface"""

    url_path = "assistants"
    files: AssistantFile

    def __init__(self, http_client: httpx.Client) -> None:
        super().__init__(http_client=http_client)
        self.files = AssistantFile(http_client=http_client)

    def create(
        self,
        *,
        model: Literal[
            "abab6-chat",
            "abab5.5-chat",
            "abab5.5-chat-240131",
            "abab5.5s-chat",
            "abab5.5s-chat-240123",
        ],
        name: Optional[str] = None,
        instructions: Optional[str] = None,
        description: Optional[str] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        file_ids: Optional[List[str]] = None,
        rolemeta: Optional[Dict[str, str]] = None,
        metadata: Optional[Dict[str, str]] = None,
        t2a_option: Optional[Dict[str, str]] = None,
    ) -> AssistantCreateResponse:
        """
        Create a new assistant

        Args:
            model (Literal["abab6-chat", "abab5.5-chat", "abab5.5s-chat",
                "abab5.5-chat-240131", "abab5.5s-chat-240123"]):
                The model to use for the assistant
            name (Optional[str], optional): The name of the assistant. Defaults to None.
            instructions (Optional[str], optional):
                The instructions for the assistant. Defaults to None.
            description (Optional[str], optional):
                The description of the assistant. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the assistant. Defaults to None.
            file_ids (Optional[List[str]], optional):
                The file IDs to use for the assistant. Defaults to None.
            rolemeta (Optional[Dict[str, str]], optional):
                The rolemeta to use for the assistant. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata to use for the assistant. Defaults to None.
            t2a_option (Optional[Dict[str, str]], optional):
                The t2a_option to use for the assistant. Defaults to None.

        Returns:
            AssistantCreateResponse:
                The response of the API containing the created assistant
        """
        json_body: Dict[str, Any] = {"model": model}

        if name:
            json_body["name"] = name
        if instructions:
            json_body["instructions"] = instructions
        if description:
            json_body["description"] = description
        if tools:
            json_body["tools"] = tools
        if file_ids:
            json_body["file_ids"] = file_ids
        if rolemeta:
            json_body["rolemeta"] = rolemeta
        if metadata:
            json_body["metadata"] = metadata
        if t2a_option:
            json_body["t2a_option"] = t2a_option

        resp = self.client.post(url=f"{self.url_path}/create", json=json_body)

        return AssistantCreateResponse(**resp.json())

    def retrieve(self, assistant_id: str) -> AssistantRetrieveResponse:
        """
        Retrieve an assistant

        Args:
            assistant_id (str): The ID of the assistant to retrieve

        Returns:
            Assistant: The response of the API containing the retrieved assistant
        """
        resp = self.client.get(
            url=f"{self.url_path}/retrieve", params={"assistant_id": assistant_id}
        )

        return AssistantRetrieveResponse(**resp.json())

    def update(
        self,
        assistant_id: str,
        *,
        model: Literal[
            "abab6-chat",
            "abab5.5-chat",
            "abab5.5-chat-240131",
            "abab5.5s-chat",
            "abab5.5s-chat-240123",
        ],
        name: str,
        instructions: str,
        description: Optional[str] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        file_ids: Optional[List[str]] = None,
        rolemeta: Optional[Dict[str, str]] = None,
        metadata: Optional[Dict[str, str]] = None,
        t2a_option: Optional[Dict[str, str]] = None,
    ) -> AssistantUpdateResponse:
        """
        Update an assistant

        Args:
            assistant_id (str): The ID of the assistant to update
            model (Literal["abab6-chat", "abab5.5-chat", "abab5.5s-chat",
                "abab5.5-chat-240131", "abab5.5s-chat-240123"]):
                The model to use for the assistant
            name (str): The name of the assistant
            instructions (str): The instructions for the assistant
            description (Optional[str], optional):
                The description of the assistant. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the assistant. Defaults to None.
            file_ids (Optional[List[str]], optional):
                The file IDs to use for the assistant. Defaults to None.
            rolemeta (Optional[Dict[str, str]], optional):
                The rolemeta to use for the assistant. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata to use for the assistant. Defaults to None.
            t2a_option (Optional[Dict[str, str]], optional):
                The t2a_option to use for the assistant. Defaults to None.

        Returns:
            AssistantUpdateResponse:
                The response of the API containing the updated assistant
        """
        json_body: Dict[str, Any] = {
            "id": assistant_id,
            "model": model,
            "name": name,
            "instructions": instructions,
        }

        if description:
            json_body["description"] = description
        if tools:
            json_body["tools"] = tools
        if file_ids:
            json_body["file_ids"] = file_ids
        if rolemeta:
            json_body["rolemeta"] = rolemeta
        if metadata:
            json_body["metadata"] = metadata
        if t2a_option:
            json_body["t2a_option"] = t2a_option

        resp = self.client.post(url=f"{self.url_path}/modify", json=json_body)

        return AssistantUpdateResponse(**resp.json())

    def delete(self, assistant_id: str) -> AssistantDeleteResponse:
        """
        Delete an assistant

        Args:
            assistant_id (str): The ID of the assistant to delete

        Returns:
            AssistantDeleteResponse: The response from the API
        """
        resp = self.client.post(
            url=f"{self.url_path}/delete", json={"assistant_id": assistant_id}
        )

        return AssistantDeleteResponse(**resp.json())

    def list(
        self,
        limit: int = 20,
        order: Literal["asc", "desc"] = "desc",
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> AssistantListResponse:
        """
        List all assistants

        Args:
            limit (int): The number of assistants to return. Defaults to 20.
            order (str):
                The order of the list, could be "asc" or "desc". Defaults to "desc".
            after (Optional[str], optional):
                The ID of the assistant to start after. Defaults to None.
            before (Optional[str], optional):
                The ID of the assistant to end before. Defaults to None.

        Returns:
            AssistantListResponse:
                The response from the API containing the list of assistants
        """
        params = {"limit": limit, "order": order}

        if after:
            params["after"] = after
        if before:
            params["before"] = before

        resp = self.client.get(url=f"{self.url_path}/list", params=params)

        return AssistantListResponse(**resp.json())


class AsyncAssistant(BaseAsyncInterface, Assistant):
    """Asynchronous Assistants interface"""

    files: AsyncAssistantFile

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        super().__init__(http_client=http_client)
        self.files = AsyncAssistantFile(http_client=http_client)

    async def create(
        self,
        *,
        model: Literal[
            "abab6-chat",
            "abab5.5-chat",
            "abab5.5-chat-240131",
            "abab5.5s-chat",
            "abab5.5s-chat-240123",
        ],
        name: Optional[str] = None,
        instructions: Optional[str] = None,
        description: Optional[str] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        file_ids: Optional[List[str]] = None,
        rolemeta: Optional[Dict[str, str]] = None,
        metadata: Optional[Dict[str, str]] = None,
        t2a_option: Optional[Dict[str, str]] = None,
    ) -> AssistantCreateResponse:
        """
        Create a new assistant

        Args:
            model (Literal["abab6-chat", "abab5.5-chat", "abab5.5s-chat",
                "abab5.5-chat-240131", "abab5.5s-chat-240123"]):
                The model to use for the assistant
            name (Optional[str], optional): The name of the assistant. Defaults to None.
            instructions (Optional[str], optional):
                The instructions for the assistant. Defaults to None.
            description (Optional[str], optional):
                The description of the assistant. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the assistant. Defaults to None.
            file_ids (Optional[List[str]], optional):
                The file IDs to use for the assistant. Defaults to None.
            rolemeta (Optional[Dict[str, str]], optional):
                The rolemeta to use for the assistant. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata to use for the assistant. Defaults to None.
            t2a_option (Optional[Dict[str, str]], optional):
                The t2a_option to use for the assistant. Defaults to None.

        Returns:
            AssistantCreateResponse:
                The response of the API containing the created assistant
        """
        json_body: Dict[str, Any] = {"model": model}

        if name:
            json_body["name"] = name
        if instructions:
            json_body["instructions"] = instructions
        if description:
            json_body["description"] = description
        if tools:
            json_body["tools"] = tools
        if file_ids:
            json_body["file_ids"] = file_ids
        if rolemeta:
            json_body["rolemeta"] = rolemeta
        if metadata:
            json_body["metadata"] = metadata
        if t2a_option:
            json_body["t2a_option"] = t2a_option

        resp = await self.client.post(url=f"{self.url_path}/create", json=json_body)

        return AssistantCreateResponse(**resp.json())

    async def retrieve(self, assistant_id: str) -> AssistantRetrieveResponse:
        """
        Retrieve an assistant

        Args:
            assistant_id (str): The ID of the assistant to retrieve

        Returns:
            Assistant: The response of the API containing the retrieved assistant
        """
        resp = await self.client.get(
            url=f"{self.url_path}/retrieve", params={"assistant_id": assistant_id}
        )

        return AssistantRetrieveResponse(**resp.json())

    async def update(
        self,
        assistant_id: str,
        *,
        model: Literal[
            "abab6-chat",
            "abab5.5-chat",
            "abab5.5-chat-240131",
            "abab5.5s-chat",
            "abab5.5s-chat-240123",
        ],
        name: str,
        instructions: str,
        description: Optional[str] = None,
        tools: Optional[List[Dict[str, Union[str, Dict]]]] = None,
        file_ids: Optional[List[str]] = None,
        rolemeta: Optional[Dict[str, str]] = None,
        metadata: Optional[Dict[str, str]] = None,
        t2a_option: Optional[Dict[str, str]] = None,
    ) -> AssistantUpdateResponse:
        """
        Update an assistant

        Args:
            assistant_id (str): The ID of the assistant to update
            model (Literal["abab6-chat", "abab5.5-chat", "abab5.5s-chat",
                "abab5.5-chat-240131", "abab5.5s-chat-240123"]):
                The model to use for the assistant
            name (str): The name of the assistant
            instructions (str): The instructions for the assistant
            description (Optional[str], optional):
                The description of the assistant. Defaults to None.
            tools (Optional[List[Dict[str, Union[str, Dict]]]], optional):
                The tools to use for the assistant. Defaults to None.
            file_ids (Optional[List[str]], optional):
                The file IDs to use for the assistant. Defaults to None.
            rolemeta (Optional[Dict[str, str]], optional):
                The rolemeta to use for the assistant. Defaults to None.
            metadata (Optional[Dict[str, str]], optional):
                The metadata to use for the assistant. Defaults to None.
            t2a_option (Optional[Dict[str, str]], optional):
                The t2a_option to use for the assistant. Defaults to None.

        Returns:
            AssistantUpdateResponse:
                The response of the API containing the updated assistant
        """
        json_body: Dict[str, Any] = {
            "id": assistant_id,
            "model": model,
            "name": name,
            "instructions": instructions,
        }

        if description:
            json_body["description"] = description
        if tools:
            json_body["tools"] = tools
        if file_ids:
            json_body["file_ids"] = file_ids
        if rolemeta:
            json_body["rolemeta"] = rolemeta
        if metadata:
            json_body["metadata"] = metadata
        if t2a_option:
            json_body["t2a_option"] = t2a_option

        resp = await self.client.post(url=f"{self.url_path}/modify", json=json_body)

        return AssistantUpdateResponse(**resp.json())

    async def delete(self, assistant_id: str) -> AssistantDeleteResponse:
        """
        Delete an assistant

        Args:
            assistant_id (str): The ID of the assistant to delete

        Returns:
            AssistantDeleteResponse: The response from the API
        """
        resp = await self.client.post(
            url=f"{self.url_path}/delete", json={"assistant_id": assistant_id}
        )

        return AssistantDeleteResponse(**resp.json())

    async def list(
        self,
        limit: int = 20,
        order: Literal["asc", "desc"] = "desc",
        after: Optional[str] = None,
        before: Optional[str] = None,
    ) -> AssistantListResponse:
        """
        List all assistants

        Args:
            limit (int): The number of assistants to return. Defaults to 20.
            order (str):
                The order of the list, could be "asc" or "desc". Defaults to "desc".
            after (Optional[str], optional):
                The ID of the assistant to start after. Defaults to None.
            before (Optional[str], optional):
                The ID of the assistant to end before. Defaults to None.

        Returns:
            AssistantListResponse:
                The response from the API containing the list of assistants
        """
        params = {"limit": limit, "order": order}

        if after:
            params["after"] = after
        if before:
            params["before"] = before

        resp = await self.client.get(url=f"{self.url_path}/list", params=params)

        return AssistantListResponse(**resp.json())



"""fine_tuning.py"""

from typing import Dict, Literal, Optional, Union

import httpx

from minimax_client.entities.common import BareResponse
from minimax_client.entities.fine_tuning import (
    FineTuningJobCreateResponse,
    FineTuningJobEventListResponse,
    FineTuningJobListResponse,
    FineTuningModelListResponse,
    FineTuningModelRetrieveResponse,
)
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class FineTuningJob(BaseSyncInterface):
    """Synchronous Fine Tuning Jobs interface"""

    url_path = ""

    def create(
        self,
        *,
        model: Literal["abab5.5-chat-240119", "abab5.5s-chat-240123"],
        training_file: int,
        validation_file: Optional[int] = None,
        hyperparameters: Optional[Dict[str, Union[int, float]]] = None,
        suffix: Optional[str] = None,
    ) -> FineTuningJobCreateResponse:
        """
        Create a new fine tuning job

        Args:
            model (Literal["abab5.5-chat-240119", "abab5.5s-chat-240123"]):
                The model to fine tune
            training_file (int): The ID of the training file
            validation_file (Optional[int], optional): The ID of the validation file.
                Defaults to None.
            hyperparameters (Optional[Dict[str, Union[int, float]]], optional):
                The hyperparameters to fine tune. Defaults to None.
            suffix (Optional[str], optional): The suffix to use for the fine tuning job.
                Defaults to None.

        Returns:
            FineTuningJobCreateResponse: The created fine tuning job
        """
        json_body = {
            "model": model,
            "training_file": training_file,
        }

        if validation_file:
            json_body["validation_file"] = validation_file

        if hyperparameters:
            json_body["hyperparameters"] = hyperparameters

        if suffix:
            json_body["suffix"] = suffix

        resp = self.client.post(url="create_finetune_job", json=json_body)

        return FineTuningJobCreateResponse(**resp.json())

    def list(
        self, limit: int, after: Optional[str] = None
    ) -> FineTuningJobListResponse:
        """
        List all fine tuning jobs

        Args:
            limit (int): The number of fine tuning jobs to return
            after (Optional[str], optional):
                The ID of the fine tuning job to start after. Defaults to None.

        Returns:
            FineTuningJobListResponse: The list of fine tuning jobs
        """
        json_body: Dict[str, Union[int, str]] = {"limit": limit}

        if after:
            json_body["after"] = after

        resp = self.client.post(url="list_finetune_job", json=json_body)

        return FineTuningJobListResponse(**resp.json())

    def retrieve(self, fine_tuning_job_id: str) -> FineTuningJobCreateResponse:
        """
        Retrieve a fine tuning job

        Args:
            fine_tuning_job_id (str): The ID of the fine tuning job to retrieve

        Returns:
            FineTuningJobCreateResponse: The retrieved fine tuning job
        """
        json_body = {"fine_tuning_job_id": fine_tuning_job_id}

        resp = self.client.post(url="retrieve_finetune_job", json=json_body)

        return FineTuningJobCreateResponse(**resp.json())

    def cancel(self, fine_tuning_job_id: str) -> BareResponse:
        """
        Cancel a fine tuning job

        Args:
            fine_tuning_job_id (str): The ID of the fine tuning job to cancel

        Returns:
            BareResponse: The response from the API
        """
        json_body = {"fine_tuning_job_id": fine_tuning_job_id}

        resp = self.client.post(url="delete_finetune_job", json=json_body)

        return BareResponse(**resp.json())

    def list_events(
        self, fine_tuning_job_id: str, limit: int, after: Optional[str] = None
    ) -> FineTuningJobEventListResponse:
        """
        List events for a fine tuning job

        Args:
            fine_tuning_job_id (str): The ID of the fine tuning job
            limit (int): The number of events to return
            after (Optional[str], optional):
                The ID of the event to start after. Defaults to None.

        Returns:
            FineTuningJobEventListResponse: The list of events
        """
        json_body = {"fine_tuning_job_id": fine_tuning_job_id, "limit": limit}

        if after:
            json_body["after"] = after

        resp = self.client.post(url="list_finetune_event", json=json_body)

        return FineTuningJobEventListResponse(**resp.json())


class AsyncFineTuningJob(BaseAsyncInterface, FineTuningJob):
    """Asynchronous Fine Tuning Jobs interface"""

    async def create(
        self,
        *,
        model: Literal["abab5.5-chat-240119", "abab5.5s-chat-240123"],
        training_file: int,
        validation_file: Optional[int] = None,
        hyperparameters: Optional[Dict[str, Union[int, float]]] = None,
        suffix: Optional[str] = None,
    ) -> FineTuningJobCreateResponse:
        """
        Create a new fine tuning job

        Args:
            model (Literal["abab5.5-chat-240119", "abab5.5s-chat-240123"]):
                The model to fine tune
            training_file (int): The ID of the training file
            validation_file (Optional[int], optional): The ID of the validation file.
                Defaults to None.
            hyperparameters (Optional[Dict[str, Union[int, float]]], optional):
                The hyperparameters to fine tune. Defaults to None.
            suffix (Optional[str], optional): The suffix to use for the fine tuning job.
                Defaults to None.

        Returns:
            FineTuningJobCreateResponse: The created fine tuning job
        """
        json_body = {
            "model": model,
            "training_file": training_file,
        }

        if validation_file:
            json_body["validation_file"] = validation_file

        if hyperparameters:
            json_body["hyperparameters"] = hyperparameters

        if suffix:
            json_body["suffix"] = suffix

        resp = await self.client.post(url="create_finetune_job", json=json_body)

        return FineTuningJobCreateResponse(**resp.json())

    async def list(
        self, limit: int, after: Optional[str] = None
    ) -> FineTuningJobListResponse:
        """
        List all fine tuning jobs

        Args:
            limit (int): The number of fine tuning jobs to return
            after (Optional[str], optional):
                The ID of the fine tuning job to start after. Defaults to None.

        Returns:
            FineTuningJobListResponse: The list of fine tuning jobs
        """
        json_body: Dict[str, Union[int, str]] = {"limit": limit}

        if after:
            json_body["after"] = after

        resp = await self.client.post(url="list_finetune_job", json=json_body)

        return FineTuningJobListResponse(**resp.json())

    async def retrieve(self, fine_tuning_job_id: str) -> FineTuningJobCreateResponse:
        """
        Retrieve a fine tuning job

        Args:
            fine_tuning_job_id (str): The ID of the fine tuning job to retrieve

        Returns:
            FineTuningJobCreateResponse: The retrieved fine tuning job
        """
        json_body = {"fine_tuning_job_id": fine_tuning_job_id}

        resp = await self.client.post(url="retrieve_finetune_job", json=json_body)

        return FineTuningJobCreateResponse(**resp.json())

    async def cancel(self, fine_tuning_job_id: str) -> BareResponse:
        """
        Cancel a fine tuning job

        Args:
            fine_tuning_job_id (str): The ID of the fine tuning job to cancel

        Returns:
            BareResponse: The response from the API
        """
        json_body = {"fine_tuning_job_id": fine_tuning_job_id}

        resp = await self.client.post(url="delete_finetune_job", json=json_body)

        return BareResponse(**resp.json())

    async def list_events(
        self, fine_tuning_job_id: str, limit: int, after: Optional[str] = None
    ) -> FineTuningJobEventListResponse:
        """
        List events for a fine tuning job

        Args:
            fine_tuning_job_id (str): The ID of the fine tuning job
            limit (int): The number of events to return
            after (Optional[str], optional):
                The ID of the event to start after. Defaults to None.

        Returns:
            FineTuningJobEventListResponse: The list of events
        """
        json_body = {"fine_tuning_job_id": fine_tuning_job_id, "limit": limit}

        if after:
            json_body["after"] = after

        resp = await self.client.post(url="list_finetune_event", json=json_body)

        return FineTuningJobEventListResponse(**resp.json())


class FineTuning:
    """Synchronous Fine Tuning interface"""

    jobs: FineTuningJob

    def __init__(self, http_client: httpx.Client) -> None:
        self.jobs = FineTuningJob(http_client=http_client)


class AsyncFineTuning:
    """Asynchronous Fine Tuning interface"""

    jobs: AsyncFineTuningJob

    def __init__(self, http_client: httpx.AsyncClient) -> None:
        self.jobs = AsyncFineTuningJob(http_client=http_client)


class Model(BaseSyncInterface):
    """Synchronous Model interface"""

    url_path = ""

    def list(self) -> FineTuningModelListResponse:
        """
        List all fine tuning models

        Returns:
            FineTuningModelListResponse: The list of fine tuning models
        """
        resp = self.client.post(url="list_finetune_model")

        return FineTuningModelListResponse(**resp.json())

    def retrieve(self, model: str) -> FineTuningModelRetrieveResponse:
        """
        Retrieve a fine tuning model

        Args:
            model (str): The ID of the fine tuning model to retrieve

        Returns:
            FineTuningModelRetrieveResponse: The retrieved fine tuning model
        """
        json_body = {"model_id": model}

        resp = self.client.post(url="retrieve_finetune_model", json=json_body)

        return FineTuningModelRetrieveResponse(**resp.json())

    def delete(self, model: str) -> BareResponse:
        """
        Delete a fine tuning model

        Args:
            model (str): The ID of the fine tuning model to delete

        Returns:
            BareResponse: The response from the API
        """
        json_body = {"model_id": model}

        resp = self.client.post(url="delete_finetune_model", json=json_body)

        return BareResponse(**resp.json())


class AsyncModel(BaseAsyncInterface, Model):
    """Asynchronous Model interface"""

    async def list(self) -> FineTuningModelListResponse:
        """
        List all fine tuning models

        Returns:
            FineTuningModelListResponse: The list of fine tuning models
        """
        resp = await self.client.post(url="list_finetune_model")

        return FineTuningModelListResponse(**resp.json())

    async def retrieve(self, model: str) -> FineTuningModelRetrieveResponse:
        """
        Retrieve a fine tuning model

        Args:
            model (str): The ID of the fine tuning model to retrieve

        Returns:
            FineTuningModelRetrieveResponse: The retrieved fine tuning model
        """
        json_body = {"model_id": model}

        resp = await self.client.post(url="retrieve_finetune_model", json=json_body)

        return FineTuningModelRetrieveResponse(**resp.json())

    async def delete(self, model: str) -> BareResponse:
        """
        Delete a fine tuning model

        Args:
            model (str): The ID of the fine tuning model to delete

        Returns:
            BareResponse: The response from the API
        """
        json_body = {"model_id": model}

        resp = await self.client.post(url="delete_finetune_model", json=json_body)

        return BareResponse(**resp.json())



"""client.py"""

import asyncio
import os
from typing import Optional

import httpx
from dotenv import find_dotenv, load_dotenv

from minimax_client.interfaces.assistant import Assistant, AsyncAssistant
from minimax_client.interfaces.audio import AsyncAudio, Audio
from minimax_client.interfaces.chat_completion import AsyncChat, Chat
from minimax_client.interfaces.embedding import AsyncEmbedding, Embedding
from minimax_client.interfaces.file import AsyncFiles, Files
from minimax_client.interfaces.fine_tuning import (
    AsyncFineTuning,
    AsyncModel,
    FineTuning,
    Model,
)
from minimax_client.interfaces.thread import AsyncThreads, Threads

BASE_URL = "https://api.minimax.chat/v1"


class BaseMiniMaxClient:
    """MiniMax client base class"""

    api_key: str
    group_id: str
    timeout: float

    def __init__(
        self,
        api_key: Optional[str] = None,
        group_id: Optional[str] = None,
        timeout: float = 60,
    ) -> None:
        if not api_key:
            api_key = self._get_api_key_from_env()

        if not group_id:
            group_id = os.getenv("MINIMAX_GROUP_ID", "")

        self.api_key = api_key
        self.group_id = group_id
        self.timeout = timeout
        self.http_client = self._get_http_client()

    def _get_api_key_from_env(self) -> str:
        """
        Retrieves the MiniMax API key from the environment.

        First it tries to find a `.env` file in the current
        directory or any of its parent directories. If such a file is found,
        it loads the environment variables from it.

        Then, it retrieves the `MINIMAX_API_KEY` environment variable. If it
        contains a valid API key, it returns it. Otherwise, it raises a
        `ValueError`.

        Returns:
            str: The MiniMax API key.
        """
        env_file = find_dotenv()

        if env_file:
            load_dotenv(dotenv_path=env_file)

        api_key = os.getenv("MINIMAX_API_KEY")

        if not api_key:
            raise ValueError("A valid MiniMax API key must be provided!")

        return api_key

    def _get_http_client(self):
        raise NotImplementedError


class MiniMax(BaseMiniMaxClient):
    """MiniMax client"""

    http_client: httpx.Client
    assistants: Assistant
    audio: Audio
    chat: Chat
    embeddings: Embedding
    files: Files
    fine_tuning: FineTuning
    model: Model
    threads: Threads

    def __init__(self, *, api_key: Optional[str] = None, timeout: float = 60) -> None:
        super().__init__(api_key=api_key, timeout=timeout)
        self.assistants = Assistant(http_client=self.http_client)
        self.audio = Audio(http_client=self.http_client)
        self.chat = Chat(http_client=self.http_client)
        self.embeddings = Embedding(http_client=self.http_client)
        self.files = Files(http_client=self.http_client)
        self.fine_tuning = FineTuning(http_client=self.http_client)
        self.model = Model(http_client=self.http_client)
        self.threads = Threads(http_client=self.http_client)

    def __del__(self) -> None:
        """Closes the HTTP client if it is not already closed."""
        if hasattr(self, "http_client") and not self.http_client.is_closed:
            self.http_client.close()

    def _get_http_client(self) -> httpx.Client:
        """
        Returns a synchronous HTTP client with the base URL and authentication
        header configured.

        The client is configured to send requests to the MiniMax API with the
        provided API key.

        Returns:
            httpx.Client: The synchronous HTTP client.
        """
        return httpx.Client(
            base_url=BASE_URL,
            headers={"Authorization": f"Bearer {self.api_key}"},
            params={"GroupId": self.group_id},
            timeout=self.timeout,
        )


class AsyncMiniMax(BaseMiniMaxClient):
    """MiniMax async client"""

    http_client: httpx.AsyncClient
    assistants: AsyncAssistant
    audio: AsyncAudio
    chat: AsyncChat
    embeddings: AsyncEmbedding
    files: AsyncFiles
    fine_tuning: AsyncFineTuning
    model: AsyncModel
    threads: AsyncThreads

    def __init__(self, *, api_key: Optional[str] = None, timeout: float = 60) -> None:
        super().__init__(api_key=api_key, timeout=timeout)
        self.assistants = AsyncAssistant(http_client=self.http_client)
        self.audio = AsyncAudio(http_client=self.http_client)
        self.chat = AsyncChat(http_client=self.http_client)
        self.embeddings = AsyncEmbedding(http_client=self.http_client)
        self.files = AsyncFiles(http_client=self.http_client)
        self.fine_tuning = AsyncFineTuning(http_client=self.http_client)
        self.model = AsyncModel(http_client=self.http_client)
        self.threads = AsyncThreads(http_client=self.http_client)

    def __del__(self) -> None:
        async def __del_client() -> None:
            """
            Closes the async HTTP client if it is not already closed.

            This coroutine is called when the AsyncMiniMax instance is garbage
            collected. It asynchronously closes the HTTP client if it is not
            already closed.
            """
            if hasattr(self, "http_client") and not self.http_client.is_closed:
                await self.http_client.aclose()

        # Create a task to close the HTTP client. This task is scheduled to run
        # asyncio's event loop, but it does not block the interpreter from
        # exiting.
        asyncio.create_task(__del_client())

    def _get_http_client(self) -> httpx.AsyncClient:
        """
        Returns a new asynchronous HTTP client with the base URL and
        authentication header configured.

        The client is configured to send requests to the MiniMax API with the
        provided API key.

        Returns:
            httpx.AsyncClient: The asynchronous HTTP client.
        """
        return httpx.AsyncClient(
            base_url=BASE_URL,
            headers={"Authorization": f"Bearer {self.api_key}"},
            timeout=self.timeout,
        )



"""test_client.py"""

import os
import unittest

import httpx

from minimax_client import AsyncMiniMax, MiniMax
from minimax_client.interfaces.chat_completion import AsyncChat


class TestMiniMax(unittest.TestCase):
    """Unit tests for the MiniMax client."""

    def test_init_without_api_key(self):
        """Test that an error is raised if no API key is provided."""
        with self.assertRaises(ValueError):
            if os.environ.get("MINIMAX_API_KEY"):
                del os.environ["MINIMAX_API_KEY"]

            MiniMax()

    def test_init_with_valid_api_key(self):
        """Test that the client is initialized with a valid API key."""
        client = MiniMax(api_key="valid_api_key")
        self.assertEqual(client.api_key, "valid_api_key")

    def test_del_closes_http_client(self):
        """Test that the __del__ method closes the HTTP client."""
        client = MiniMax(api_key="valid_api_key")
        client.http_client.close()
        self.assertTrue(client.http_client.is_closed)


class TestAsyncMiniMax(unittest.IsolatedAsyncioTestCase):
    """Unit tests for AsyncMiniMax"""

    async def test_constructor(self) -> None:
        """Test that the AsyncMiniMax constructor sets the API key and creates
        an AsyncChat instance with the HTTP client"""
        api_key = "test_api_key"
        async_minimax = AsyncMiniMax(api_key=api_key)
        self.assertEqual(async_minimax.api_key, api_key)
        self.assertIsInstance(async_minimax.chat, AsyncChat)
        self.assertIs(async_minimax.chat.completions.client, async_minimax.http_client)

    async def test_del_closes_http_client(self) -> None:
        """Test that the __del__ method closes the HTTP client"""
        async_minimax = AsyncMiniMax(api_key="valid_api_key")
        await async_minimax.http_client.aclose()
        self.assertTrue(async_minimax.http_client.is_closed)

    async def test_get_http_client(self) -> None:
        """Test that _get_http_client returns a new AsyncClient instance with
        the base URL and authorization header set"""
        api_key = "test_api_key"
        async_minimax = AsyncMiniMax(api_key=api_key)
        http_client = async_minimax._get_http_client()
        self.assertIsInstance(http_client, httpx.AsyncClient)
        self.assertEqual(http_client.headers["Authorization"], f"Bearer {api_key}")



"""common.py"""

from pydantic import BaseModel, NonNegativeInt


class BaseResp(BaseModel):
    """Base Response

    status_code:
        1000: Unknown Error
        1001: Timeout
        1002: RPM limit reached
        1004: Authorization Failure
        1008: Balance Insufficient
        1013: Internal Service Error
        1026: Input incorrect
        1027: Output incorrect
        1039: TPM limit reached
        2013: Input format info incorrect
    """

    status_code: NonNegativeInt
    status_msg: str


class BareResponse(BaseModel):
    """Bare Response with only 'base_resp'"""

    base_resp: BaseResp



"""embedding.py"""

from typing import List

from pydantic import NonNegativeInt

from minimax_client.entities.common import BareResponse


class EmbeddingResponse(BareResponse):
    """Embeddings Response"""

    vectors: List[List[float]]
    total_tokens: NonNegativeInt



"""audio.py"""

import json
from pathlib import Path
from typing import AsyncGenerator, Dict, Generator, List, Literal, Optional, Union

from minimax_client.entities.audio import (
    AudioSpeechErrorResponse,
    AudioSpeechLargeResponse,
    AudioSpeechLargeStatusResponse,
    AudioSpeechProResponse,
    AudioSpeechStreamResponse,
    VoiceCloningResponse,
)
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class Audio(BaseSyncInterface):
    """Synchronous Audio (T2A) interface"""

    url_path = ""

    def speech(
        self,
        *,
        text: str,
        model: Literal["speech-01", "speech-02"],
        voice_id: Optional[str] = None,
        timber_weights: Optional[List[Dict[str, Union[str, int]]]] = None,
        speed: float = 1.0,
        vol: float = 1.0,
        output_format: Literal["mp3", "wav", "pcm", "flac", "aac"] = "mp3",
        pitch: int = 0,
        char_to_pitch: Optional[List[str]] = None,
    ) -> Union[AudioSpeechErrorResponse, bytes]:
        """
        Text to Speech. Maximum characters to synthesize: 500

        Args:
            text (str): The text to synthesize. Should be < 500 characters.
            model (Literal["speech-01", "speech-02"]):
                The model to use.
                "speech-01" is suited in Chinese cases;
                "speech-02" supports Chinese, English, CN/EN mix, Japanese and Korean.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            timber_weights (Optional[List[Dict[str, Union[str, int]]]], optional):
                Weights info of mixture of voices.
                If `timber_weights` is specified, `voice_id` will be ignored.
                Defaults to None.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            output_format (Literal["mp3", "wav", "pcm", "flac", "aac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            Union[AudioSpeechErrorResponse, bytes]: The response from the API
        """
        json_body = {
            "text": text,
            "model": model,
            "speed": speed,
            "vol": vol,
            "output_format": output_format,
            "pitch": pitch,
        }

        if timber_weights:
            json_body["timber_weights"] = timber_weights
        elif voice_id:
            json_body["voice_id"] = voice_id

        if char_to_pitch:
            json_body["char_to_pitch"] = char_to_pitch

        resp = self.client.post(url="text_to_speech", json=json_body)

        if resp.headers["Content-Type"] != "audio/mpeg":
            return AudioSpeechErrorResponse(**resp.json())

        return resp.content

    def speech_pro(
        self,
        *,
        text: str,
        model: Literal["speech-01", "speech-02"],
        voice_id: Optional[str] = None,
        timber_weights: Optional[List[Dict[str, Union[str, int]]]] = None,
        speed: float = 1.0,
        vol: float = 1.0,
        output_format: Literal["mp3", "wav", "pcm", "flac", "aac"] = "mp3",
        pitch: int = 0,
        audio_sample_rate: Literal[16_000, 24_000, 32_000] = 32_000,
        bitrate: Literal[32_000, 64_000, 128_000] = 128_000,
        char_to_pitch: Optional[List[str]] = None,
    ) -> AudioSpeechProResponse:
        """
        Text to Speech Pro. Maximum characters to synthesize: 50,000

        Args:
            text (str): The text to synthesize. Should be < 50,000 characters.
            model (Literal["speech-01", "speech-02"]):
                The model to use.
                "speech-01" is suited in Chinese cases;
                "speech-02" supports Chinese, English, CN/EN mix, Japanese and Korean.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            timber_weights (Optional[List[Dict[str, Union[str, int]]]], optional):
                Weights info of mixture of voices.
                If `timber_weights` is specified, `voice_id` will be ignored.
                Defaults to None.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            output_format (Literal["mp3", "wav", "pcm", "flac", "aac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            audio_sample_rate (Literal[16_000, 24_000, 32_000]):
                The sample rate of the synthesized voice. Defaults to 32,000.
            bitrate (Literal[32_000, 64_000, 128_000]):
                The bitrate of the synthesized voice. Defaults to 128,000.
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            AudoSpeechProResponse: The response from the API
        """
        json_body = {
            "text": text,
            "model": model,
            "speed": speed,
            "vol": vol,
            "output_format": output_format,
            "pitch": pitch,
            "audio_sample_rate": audio_sample_rate,
            "bitrate": bitrate,
        }

        if timber_weights:
            json_body["timber_weights"] = timber_weights
        elif voice_id:
            json_body["voice_id"] = voice_id

        if char_to_pitch:
            json_body["char_to_pitch"] = char_to_pitch

        resp = self.client.post(url="t2a_pro", json=json_body)

        return AudioSpeechProResponse(**resp.json())

    def speech_large(
        self,
        *,
        file_path: Union[str, Path],
        model: Literal["speech-01"],
        voice_id: str,
        speed: float = 1.0,
        vol: float = 1.0,
        output_format: Literal["mp3", "wav", "pcm", "flac", "aac"] = "mp3",
        pitch: int = 0,
        audio_sample_rate: Literal[16_000, 24_000] = 24_000,
        bitrate: Literal[32_900, 64_900, 128_900] = 64_900,
        char_to_pitch: Optional[List[str]] = None,
    ) -> AudioSpeechLargeResponse:
        """
        Text to Speech Large. Maximum characters to synthesize: 10,000,000
        Asynchronous request

        Args:
            file_path (Union[str, Path]):
                The path to a single (local) zip file containing the text to synthesize.
                The zip file should only contain txt or json files.
                The text inside should be < 10,000,000 characters.
            model (Literal["speech-01"]):
                The model to use. Only `speech-01` is supported.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            output_format (Literal["mp3", "wav", "pcm", "flac", "aac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            audio_sample_rate (Literal[16_000, 24_000]):
                The sample rate of the synthesized voice. Defaults to 24,000.
            bitrate (Literal[32_900, 64_900, 128_900]):
                The bitrate of the synthesized voice. Defaults to 64,900.
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            AudioSpeechLargeResponse: The response from the API
        """
        if isinstance(file_path, str):
            file_path = Path(file_path)

        if not file_path.exists() or not file_path.is_file():
            raise FileNotFoundError(f"{file_path} does not exist or is not a file")

        body = {
            "model": model,
            "voice_id": voice_id,
            "speed": speed,
            "vol": vol,
            "output_format": output_format,
            "pitch": pitch,
            "audio_sample_rate": audio_sample_rate,
            "bitrate": bitrate,
        }

        if char_to_pitch:
            body["char_to_pitch"] = char_to_pitch

        with file_path.open("rb") as file:
            resp = self.client.post(url="t2a_async", data=body, files={"text": file})

        return AudioSpeechLargeResponse(**resp.json())

    def speech_large_status(self, task_id: int) -> AudioSpeechLargeStatusResponse:
        """
        Retrieve the status of a Text to Speech Large request.

        Args:
            task_id (int): The task_id from the response which creates the request.

        Returns:
            AudioSpeechLargeStatusResponse: The response from the API
        """
        resp = self.client.get(
            url="https://api.minimax.chat/query/t2a_async_query",
            params={"task_id": task_id},
        )

        return AudioSpeechLargeStatusResponse(**resp.json())

    def speech_stream(
        self,
        *,
        text: str,
        model: Literal["speech-01"],
        voice_id: Optional[str] = None,
        timber_weights: Optional[List[Dict[str, Union[str, int]]]] = None,
        speed: float = 1.0,
        vol: float = 1.0,
        format: Literal["mp3", "pcm", "flac"] = "mp3",
        pitch: int = 0,
        audio_sample_rate: Literal[16_000, 24_000, 32_000] = 32_000,
        bitrate: Literal[32_000, 64_000, 128_000] = 128_000,
        char_to_pitch: Optional[List[str]] = None,
    ) -> Generator[AudioSpeechStreamResponse, None, None]:
        """
        Text to Speech Stream. Maximum characters to synthesize: 500

        Args:
            text (str): The text to synthesize.
            model (Literal["speech-01"]):
                The model to use. Only `speech-01` is supported.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            timber_weights (Optional[List[Dict[str, Union[str, int]]]], optional):
                Weights info of mixture of voices.
                If `timber_weights` is specified, `voice_id` will be ignored.
                Defaults to None.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            format (Literal["mp3", "pcm", "flac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            audio_sample_rate (Literal[16_000, 24_000, 32_000]):
                The sample rate of the synthesized voice. Defaults to 32,000.
            bitrate (Literal[32_000, 64_000, 128_000]):
                The bitrate of the synthesized voice. Defaults to 128,000.
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            Generator[AudioSpeechStreamResponse, None, None]: The response from the API
        """
        json_body = {
            "text": text,
            "model": model,
            "speed": speed,
            "vol": vol,
            "format": format,
            "pitch": pitch,
            "audio_sample_rate": audio_sample_rate,
            "bitrate": bitrate,
        }

        if timber_weights:
            json_body["timber_weights"] = timber_weights
            json_body["voice_id"] = ""
        elif voice_id:
            json_body["voice_id"] = voice_id

        if char_to_pitch:
            json_body["char_to_pitch"] = char_to_pitch

        with self.client.stream(
            method="POST", url="tts/stream", json=json_body
        ) as resp:
            partial_text = ""

            for data_text in resp.iter_text():
                if not data_text.startswith("data: "):
                    partial_text += data_text
                else:
                    partial_text = data_text

                if "trace_id" in partial_text:
                    data_json = json.loads(partial_text.strip().split("data: ", 2)[1])
                    yield AudioSpeechStreamResponse(**data_json)

    def voice_cloning(self, file_id: int, voice_id: str) -> VoiceCloningResponse:
        """
        Voice Cloning.
        Must have an audio file uploaded in advance and acquire the id of that file.

        Args:
            file_id (int):
                The ID of the already uploaded audio file to clone against.
                The audio file should be in mp3, m4a or wav format.
                The size of the audio file should not exceed 20 MB.
                Length of audio should be within 5 minutes and longer than 10 seconds.
            voice_id (str):
                Custom ID to identify the cloned voice.
                Must be unique (ie. not the same as any existing voice_id).
                Must be at least 8 characters long.
                Must contain both letters and numbers, and start with a letter.
                Eg. "MiniMax001".

        Returns:
            VoiceCloningResponse: The response from the API
        """
        json_body = {"file_id": file_id, "voice_id": voice_id}

        resp = self.client.post(url="voice_clone", json=json_body)

        return VoiceCloningResponse(**resp.json())


class AsyncAudio(BaseAsyncInterface, Audio):
    """Asynchronous Audio (T2A) interface"""

    async def speech(
        self,
        *,
        text: str,
        model: Literal["speech-01", "speech-02"],
        voice_id: Optional[str] = None,
        timber_weights: Optional[List[Dict[str, Union[str, int]]]] = None,
        speed: float = 1.0,
        vol: float = 1.0,
        output_format: Literal["mp3", "wav", "pcm", "flac", "aac"] = "mp3",
        pitch: int = 0,
        char_to_pitch: Optional[List[str]] = None,
    ) -> Union[AudioSpeechErrorResponse, bytes]:
        """
        Text to Speech. Maximum characters to synthesize: 500

        Args:
            text (str): The text to synthesize. Should be < 500 characters.
            model (Literal["speech-01", "speech-02"]):
                The model to use.
                "speech-01" is suited in Chinese cases;
                "speech-02" supports Chinese, English, CN/EN mix, Japanese and Korean.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            timber_weights (Optional[List[Dict[str, Union[str, int]]]], optional):
                Weights info of mixture of voices.
                If `timber_weights` is specified, `voice_id` will be ignored.
                Defaults to None.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            output_format (Literal["mp3", "wav", "pcm", "flac", "aac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            Union[AudioSpeechErrorResponse, bytes]: The response from the API
        """
        json_body = {
            "text": text,
            "model": model,
            "speed": speed,
            "vol": vol,
            "output_format": output_format,
            "pitch": pitch,
        }

        if timber_weights:
            json_body["timber_weights"] = timber_weights
        elif voice_id:
            json_body["voice_id"] = voice_id

        if char_to_pitch:
            json_body["char_to_pitch"] = char_to_pitch

        resp = await self.client.post(url="text_to_speech", json=json_body)

        if resp.headers["Content-Type"] != "audio/mpeg":
            return AudioSpeechErrorResponse(**resp.json())

        return resp.content

    async def speech_pro(
        self,
        *,
        text: str,
        model: Literal["speech-01", "speech-02"],
        voice_id: Optional[str] = None,
        timber_weights: Optional[List[Dict[str, Union[str, int]]]] = None,
        speed: float = 1.0,
        vol: float = 1.0,
        output_format: Literal["mp3", "wav", "pcm", "flac", "aac"] = "mp3",
        pitch: int = 0,
        audio_sample_rate: Literal[16_000, 24_000, 32_000] = 32_000,
        bitrate: Literal[32_000, 64_000, 128_000] = 128_000,
        char_to_pitch: Optional[List[str]] = None,
    ) -> AudioSpeechProResponse:
        """
        Text to Speech Pro. Maximum characters to synthesize: 50,000

        Args:
            text (str): The text to synthesize. Should be < 50,000 characters.
            model (Literal["speech-01", "speech-02"]):
                The model to use.
                "speech-01" is suited in Chinese cases;
                "speech-02" supports Chinese, English, CN/EN mix, Japanese and Korean.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            timber_weights (Optional[List[Dict[str, Union[str, int]]]], optional):
                Weights info of mixture of voices.
                If `timber_weights` is specified, `voice_id` will be ignored.
                Defaults to None.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            output_format (Literal["mp3", "wav", "pcm", "flac", "aac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            audio_sample_rate (Literal[16_000, 24_000, 32_000]):
                The sample rate of the synthesized voice. Defaults to 32,000.
            bitrate (Literal[32_000, 64_000, 128_000]):
                The bitrate of the synthesized voice. Defaults to 128,000.
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            AudoSpeechProResponse: The response from the API
        """
        json_body = {
            "text": text,
            "model": model,
            "speed": speed,
            "vol": vol,
            "output_format": output_format,
            "pitch": pitch,
            "audio_sample_rate": audio_sample_rate,
            "bitrate": bitrate,
        }

        if timber_weights:
            json_body["timber_weights"] = timber_weights
        elif voice_id:
            json_body["voice_id"] = voice_id

        if char_to_pitch:
            json_body["char_to_pitch"] = char_to_pitch

        resp = await self.client.post(url="t2a_pro", json=json_body)

        return AudioSpeechProResponse(**resp.json())

    async def speech_large(
        self,
        *,
        file_path: Union[str, Path],
        model: Literal["speech-01"],
        voice_id: str,
        speed: float = 1.0,
        vol: float = 1.0,
        output_format: Literal["mp3", "wav", "pcm", "flac", "aac"] = "mp3",
        pitch: int = 0,
        audio_sample_rate: Literal[16_000, 24_000] = 24_000,
        bitrate: Literal[32_900, 64_900, 128_900] = 64_900,
        char_to_pitch: Optional[List[str]] = None,
    ) -> AudioSpeechLargeResponse:
        """
        Text to Speech Large. Maximum characters to synthesize: 10,000,000
        Asynchronous request

        Args:
            file_path (Union[str, Path]):
                The path to a single (local) zip file containing the text to synthesize.
                The zip file should only contain txt or json files.
                The text inside should be < 10,000,000 characters.
            model (Literal["speech-01"]):
                The model to use. Only `speech-01` is supported.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            output_format (Literal["mp3", "wav", "pcm", "flac", "aac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            audio_sample_rate (Literal[16_000, 24_000]):
                The sample rate of the synthesized voice. Defaults to 24,000.
            bitrate (Literal[32_900, 64_900, 128_900]):
                The bitrate of the synthesized voice. Defaults to 64,900.
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            AudioSpeechLargeResponse: The response from the API
        """
        if isinstance(file_path, str):
            file_path = Path(file_path)

        if not file_path.exists() or not file_path.is_file():
            raise FileNotFoundError(f"{file_path} does not exist or is not a file")

        body = {
            "model": model,
            "voice_id": voice_id,
            "speed": speed,
            "vol": vol,
            "output_format": output_format,
            "pitch": pitch,
            "audio_sample_rate": audio_sample_rate,
            "bitrate": bitrate,
        }

        if char_to_pitch:
            body["char_to_pitch"] = char_to_pitch

        with file_path.open("rb") as file:
            resp = await self.client.post(
                url="t2a_async", data=body, files={"text": file}
            )

        return AudioSpeechLargeResponse(**resp.json())

    async def speech_large_status(self, task_id: int) -> AudioSpeechLargeStatusResponse:
        """
        Retrieve the status of a Text to Speech Large request.

        Args:
            task_id (int): The task_id from the response which creates the request.

        Returns:
            AudioSpeechLargeStatusResponse: The response from the API
        """
        resp = await self.client.get(
            url="https://api.minimax.chat/query/t2a_async_query",
            params={"task_id": task_id},
        )

        return AudioSpeechLargeStatusResponse(**resp.json())

    async def speech_stream(
        self,
        *,
        text: str,
        model: Literal["speech-01"],
        voice_id: Optional[str] = None,
        timber_weights: Optional[List[Dict[str, Union[str, int]]]] = None,
        speed: float = 1.0,
        vol: float = 1.0,
        format: Literal["mp3", "pcm", "flac"] = "mp3",
        pitch: int = 0,
        audio_sample_rate: Literal[16_000, 24_000, 32_000] = 32_000,
        bitrate: Literal[32_000, 64_000, 128_000] = 128_000,
        char_to_pitch: Optional[List[str]] = None,
    ) -> AsyncGenerator[AudioSpeechStreamResponse, None]:
        """
        Text to Speech Stream. Maximum characters to synthesize: 500

        Args:
            text (str): The text to synthesize.
            model (Literal["speech-01"]):
                The model to use. Only `speech-01` is supported.
            voice_id (Optional[str], optional):
                The ID of the voice to use. eg. "male-qn-qingse", "female-shaonv"
                Could be one of preset IDs from MiniMax, or one out of voice cloning.
            timber_weights (Optional[List[Dict[str, Union[str, int]]]], optional):
                Weights info of mixture of voices.
                If `timber_weights` is specified, `voice_id` will be ignored.
                Defaults to None.
            speed (float):
                The speed of the synthesized voice. Should be in range [0.5, 2].
                Defaults to 1.0.
            vol (float):
                The volume of the synthesized voice. Should be in range (0, 10].
                Defaults to 1.0.
            format (Literal["mp3", "pcm", "flac"]):
                The output format of the synthesized voice. Defaults to "mp3".
            pitch (int):
                The pitch of the synthesized voice. Should be in range [-12, 12].
                Defaults to 0 (which means the original pitch).
            audio_sample_rate (Literal[16_000, 24_000, 32_000]):
                The sample rate of the synthesized voice. Defaults to 32,000.
            bitrate (Literal[32_000, 64_000, 128_000]):
                The bitrate of the synthesized voice. Defaults to 128,000.
            char_to_pitch (Optional[List[str]], optional):
                List of rules to replace tones/symbols. One string per rule.
                eg. "燕少飞/(yan4)(shao3)(fei1)", "omg/oh my god", "=/等于"
                Defaults to None.

        Returns:
            AsyncGenerator[AudioSpeechStreamResponse, None]: The response from the API
        """
        json_body = {
            "text": text,
            "model": model,
            "speed": speed,
            "vol": vol,
            "format": format,
            "pitch": pitch,
            "audio_sample_rate": audio_sample_rate,
            "bitrate": bitrate,
        }

        if timber_weights:
            json_body["timber_weights"] = timber_weights
            json_body["voice_id"] = ""
        elif voice_id:
            json_body["voice_id"] = voice_id

        if char_to_pitch:
            json_body["char_to_pitch"] = char_to_pitch

        async with self.client.stream(
            method="POST", url="tts/stream", json=json_body
        ) as resp:
            partial_text = ""

            async for data_text in resp.aiter_text():
                if not data_text.startswith("data: "):
                    partial_text += data_text
                else:
                    partial_text = data_text

                if "trace_id" in partial_text:
                    data_json = json.loads(partial_text.strip().split("data: ", 2)[1])
                    yield AudioSpeechStreamResponse(**data_json)

    async def voice_cloning(self, file_id: int, voice_id: str) -> VoiceCloningResponse:
        """
        Voice Cloning.
        Must have an audio file uploaded in advance and acquire the id of that file.

        Args:
            file_id (int):
                The ID of the already uploaded audio file to clone against.
                The audio file should be in mp3, m4a or wav format.
                The size of the audio file should not exceed 20 MB.
                Length of audio should be within 5 minutes and longer than 10 seconds.
            voice_id (str):
                Custom ID to identify the cloned voice.
                Must be unique (ie. not the same as any existing voice_id).
                Must be at least 8 characters long.
                Must contain both letters and numbers, and start with a letter.
                Eg. "MiniMax001".

        Returns:
            VoiceCloningResponse: The response from the API
        """
        json_body = {"file_id": file_id, "voice_id": voice_id}

        resp = await self.client.post(url="voice_clone", json=json_body)

        return VoiceCloningResponse(**resp.json())



"""file.py"""

from pathlib import Path
from typing import Literal, Union

from minimax_client.entities.file import (
    FileCreateResponse,
    FileDeleteResponse,
    FileListResponse,
    FileRetrieveContentResponse,
    FileRetriveResponse,
)
from minimax_client.interfaces.base import BaseAsyncInterface, BaseSyncInterface


class Files(BaseSyncInterface):
    """Synchronous Files interface"""

    url_path: str = "files"

    def create(
        self,
        filepath: Union[str, Path],
        *,
        purpose: Literal[
            "retrieval",
            "fine-tune",
            "fine-tune-result",
            "voice_clone",
            "assistants",
            "role-recognition",
        ] = "retrieval",
    ) -> FileCreateResponse:
        """Upload a file

        Args:
            filepath (Union[str, Path]): The path to the file to upload
            purpose (Literal[
                "retrieval",
                "fine-tune",
                "fine-tune-result",
                "voice_clone",
                "assistants",
                "role-recognition"
            ], optional): The purpose of the file. Defaults to "retrieval".

        Raises:
            FileNotFoundError: If the file does not exist or is not a file

        Returns:
            FileCreateResponse: The response from the API
        """
        if isinstance(filepath, str):
            filepath = Path(filepath)

        if not filepath.exists() or not filepath.is_file():
            raise FileNotFoundError(f"{filepath} does not exist or is not a file")

        with filepath.open("rb") as file:
            resp = self.client.post(
                url=f"{self.url_path}/upload",
                files={"file": file},
                data={"purpose": purpose},
            )

        return FileCreateResponse(**resp.json())

    def list(
        self,
        purpose: Literal[
            "retrieval",
            "fine-tune",
            "fine-tune-result",
            "voice_clone",
            "assistants",
            "role-recognition",
        ] = "retrieval",
    ) -> FileListResponse:
        """Get the list of files under given purpose

        Args:
            purpose (Literal[
                "retrieval",
                "fine-tune",
                "fine-tune-result",
                "voice_clone",
                "assistants",
                "role-recognition"
            ], optional): The purpose of the files to list. Defaults to "retrieval".

        Returns:
            FileListResponse: The response from the API
        """
        resp = self.client.get(url=f"{self.url_path}/list", params={"purpose": purpose})

        return FileListResponse(**resp.json())

    def retrieve(self, file_id: int) -> FileRetriveResponse:
        """Retrieve general info of the given file

        Args:
            file_id (int): The ID of the file to retrieve

        Returns:
            FileRetriveResponse: The response from the API
        """
        resp = self.client.get(
            url=f"{self.url_path}/retrieve", params={"file_id": file_id}
        )

        return FileRetriveResponse(**resp.json())

    def content(self, file_id: int) -> FileRetrieveContentResponse:
        """Retrieve the content of the given file

        Args:
            file_id (int): The ID of the file to retrieve the content of

        Returns:
            FileRetrieveContentResponse: The response from the API
        """
        # TODO: clarify

        resp = self.client.get(
            url=f"{self.url_path}/retrieve_content",
            params={"file_id": file_id},
        )

        return FileRetrieveContentResponse(**resp.json())

    def delete(self, file_id: int) -> FileDeleteResponse:
        """Delete a file

        Args:
            file_id (int): The ID of the file to delete

        Returns:
            FileDeleteResponse: The response from the API
        """
        resp = self.client.post(f"{self.url_path}/delete", json={"file_id": file_id})

        return FileDeleteResponse(**resp.json())


class AsyncFiles(BaseAsyncInterface, Files):
    """Asynchronous Files interface"""

    async def create(
        self,
        filepath: Union[str, Path],
        *,
        purpose: Literal[
            "retrieval",
            "fine-tune",
            "fine-tune-result",
            "voice_clone",
            "assistants",
            "role-recognition",
        ] = "retrieval",
    ) -> FileCreateResponse:
        """Upload a file

        Args:
            filepath (Union[str, Path]): The path to the file to upload
            purpose (Literal[
                "retrieval",
                "fine-tune",
                "fine-tune-result",
                "voice_clone",
                "assistants",
                "role-recognition"
            ], optional): The purpose of the file. Defaults to "retrieval".

        Raises:
            FileNotFoundError: If the file does not exist or is not a file

        Returns:
            FileCreateResponse: The response from the API
        """
        if isinstance(filepath, str):
            filepath = Path(filepath)

        if not filepath.exists() or not filepath.is_file():
            raise FileNotFoundError(f"{filepath} does not exist or is not a file")

        with filepath.open("rb") as file:
            resp = await self.client.post(
                url=f"{self.url_path}/upload",
                files={"file": file},
                data={"purpose": purpose},
            )

        return FileCreateResponse(**resp.json())

    async def list(
        self,
        purpose: Literal[
            "retrieval",
            "fine-tune",
            "fine-tune-result",
            "voice_clone",
            "assistants",
            "role-recognition",
        ] = "retrieval",
    ) -> FileListResponse:
        """Get the list of files under given purpose

        Args:
            purpose (Literal[
                "retrieval",
                "fine-tune",
                "fine-tune-result",
                "voice_clone",
                "assistants",
                "role-recognition"
            ], optional): The purpose of the files to list. Defaults to "retrieval".

        Returns:
            FileListResponse: The response from the API
        """
        resp = await self.client.get(
            url=f"{self.url_path}/list", params={"purpose": purpose}
        )

        return FileListResponse(**resp.json())

    async def retrieve(self, file_id: int) -> FileRetriveResponse:
        """Retrieve general info of the given file

        Args:
            file_id (int): The ID of the file to retrieve

        Returns:
            FileRetriveResponse: The response from the API
        """
        resp = await self.client.get(
            url=f"{self.url_path}/retrieve", params={"file_id": file_id}
        )

        return FileRetriveResponse(**resp.json())

    async def content(self, file_id: int) -> FileRetrieveContentResponse:
        """Retrieve the content of the given file

        Args:
            file_id (int): The ID of the file to retrieve the content of

        Returns:
            FileRetrieveContentResponse: The response from the API
        """
        # TODO: clarify

        resp = await self.client.get(
            url=f"{self.url_path}/retrieve_content",
            params={"file_id": file_id},
        )

        return FileRetrieveContentResponse(**resp.json())  # to be confirmed

    async def delete(self, file_id: int) -> FileDeleteResponse:
        """Delete a file

        Args:
            file_id (int): The ID of the file to delete

        Returns:
            FileDeleteResponse: The response from the API
        """
        resp = await self.client.post(
            f"{self.url_path}/delete", json={"file_id": file_id}
        )

        return FileDeleteResponse(**resp.json())
